//
//  LiveRoomViewController.m
//  DemoAlivcLive
//
//  Created by LYZ on 16/8/2.
//  Copyright © 2016年 Alibaba. All rights reserved.
//

#import "LiveRoomViewController.h"
#import <AlivcVideoChat/AlivcVideoChat.h>
#import <MediaPlayer/MediaPlayer.h>
#import "AFNetworking.h"
#import "AlivcLiveSpotView.h"
#import "MBProgressHUD.h"
#import "LiveRoomView.h"
#import "AlivcLiveAlertView.h"
#import "MNSInfoModel.h"
#import "SendMessageManager.h"
#import "RecieveMessageManager.h"

typedef NS_ENUM(NSInteger, ALIVC_LIVE_ROOM_STATUS) {
    ALIVC_LIVE_ROOM_STATUS_NONE = 0,                // 无
    ALIVC_LIVE_ROOM_STATUS_WATCHING = 1,            // 观看直播
    ALIVC_LIVE_ROOM_STATUS_CALLING = 2,             // 准备连麦中
    ALIVC_LIVE_ROOM_STATUS_CHATTING = 3,            // 连麦中
    ALIVC_LIVE_ROOM_STATUS_CHATING_CLOSING = 4,     // 结束连麦中
};

@interface LiveRoomViewController ()<UITableViewDelegate, UITableViewDataSource,RecieveMessageDelegate, AlivcLiveAlertViewDelegate>

// UI
@property (nonatomic, strong) LiveRoomView *liveRoomView;

// SDK
@property (nonatomic, strong) NSMutableDictionary *playerParam;
@property (nonatomic, strong) NSMutableDictionary *publisherParam;
@property (nonatomic, strong) AlivcVideoChatParter *mediaPlayerCall;
@property (nonatomic, strong) NSString *mainUid;

// others
@property (nonatomic, assign) ALIVC_LIVE_ROOM_STATUS liveStatus;
@property (nonatomic, strong) NSMutableArray *chatDataArray;
@property (nonatomic, strong) NSMutableArray *interruptUids;
@property (nonatomic, strong) NSTimer *timer;
@property (nonatomic, strong) NSMutableArray<LiveInviteInfo*> *invitePlayUrlArray;
@property (nonatomic, assign) BOOL isNeedReconnectIM;
@property (nonatomic, assign) BOOL isAnomalyTimeout;
@property (nonatomic, assign) UInt64 createPlayTime;
@property (nonatomic, strong) MNSInfoModel *mnsModel;
@property (nonatomic, strong) RecieveMessageManager* mRecieveManager;

@end

@implementation LiveRoomViewController

- (void)loadView {
    
    self.liveRoomView = [[LiveRoomView alloc] initWithFrame:([UIScreen mainScreen].bounds)];
    self.view = self.liveRoomView;
}

-(void) initVariable
{
    [self.liveRoomView showLoadingViewWithMessage:@"进入直播间..."];
    
    self.chatDataArray = [NSMutableArray array];
    self.invitePlayUrlArray = [NSMutableArray array];
    self.liveStatus = ALIVC_LIVE_ROOM_STATUS_WATCHING;
    
    [self addButtonActions];
    
    [self.liveRoomView setChatTableviewDataSourceDelegate:self];
    self.liveRoomView.nameLabel.text = [NSString stringWithFormat:@"%@(%@)", self.liveName, self.userUid];
    
    self.mRecieveManager = [[RecieveMessageManager alloc] init];
    self.mRecieveManager.delegate = self;
    
    self.interruptUids = [[NSMutableArray alloc] init];
    
    //发送观看请求
    [self sendJoinLiveRequest];
    
    //创建播放器
    [self createMediaPlayer];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self initVariable];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    // 检测网络
    AFNetworkReachabilityManager *manager = [AFNetworkReachabilityManager sharedManager];
    [manager startMonitoring];
    [manager setReachabilityStatusChangeBlock:^(AFNetworkReachabilityStatus status) {
        
        switch (status) {
            case AFNetworkReachabilityStatusUnknown:
            {
                AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"当前无网络" delegate:nil buttonTitles:@"OK",nil];
                [alert showInView:self.liveRoomView];
            }
                break;
            case AFNetworkReachabilityStatusNotReachable:
            {
                AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"当前无网络" delegate:nil buttonTitles:@"OK",nil];
                [alert showInView:self.liveRoomView];
            }
                break;
            case AFNetworkReachabilityStatusReachableViaWWAN:
                NSLog(@"3G|4G");
                
                // IM重连
                if (self.isNeedReconnectIM) {
                    [self sendRequestLiveChatRoomInfoWithTopicName:self.mnsModel.topic Tags:@[self.mnsModel.roomTag, self.mnsModel.userRoomTag]];
                }
                
                break;
            case AFNetworkReachabilityStatusReachableViaWiFi:
                NSLog(@"WiFi");
                
                // IM重连
                if (self.isNeedReconnectIM) {
                    [self sendRequestLiveChatRoomInfoWithTopicName:self.mnsModel.topic Tags:@[self.mnsModel.roomTag, self.mnsModel.userRoomTag]];
                }
                
                break;
            default:
                break;
        }
    }];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - MediaPlayer

/**
 *  创建player
 */
- (void)createMediaPlayer {
    
    if(self.playUrl.length == 0) {
        AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"直播间暂无直播" delegate:nil buttonTitles:@"OK",nil];
        [alert showInView:self.liveRoomView];

        return;
    }
    
    self.createPlayTime = [[NSDate date] timeIntervalSince1970] * 1000;
    self.mediaPlayerCall = [[AlivcVideoChatParter alloc] init];

    [self addVideoChatObserver];

    self.playerParam = [[NSMutableDictionary alloc] init];
    NSNumber *dropBufferDuration = [[NSNumber alloc] initWithInt:2000];
    NSNumber *downloadTimeOut = [[NSNumber alloc] initWithInt:15000];
    
    [self.playerParam setObject:dropBufferDuration forKey:ALIVC_PLAYER_PARAM_DROPBUFFERDURATION];
    [self.playerParam setObject:[NSNumber numberWithInt:1] forKey:ALIVC_PLAYER_PARAM_SCALINGMODE];
    [self.playerParam setObject:downloadTimeOut forKey:ALIVC_PLAYER_PARAM_DOWNLOADIMEOUT];
    [self.mediaPlayerCall setPlayerParam:self.playerParam];
    
    int err = [self.mediaPlayerCall startToPlay:[NSURL URLWithString:self.playUrl] view:self.liveRoomView.mediaPalyerView];
    if(err != 0) {
        NSLog(@"preprare failed,error code is %d",(int)err);
        return;
    } else {
        NSTimer *timer = [NSTimer timerWithTimeInterval:1 target:self selector:@selector(updateInfo) userInfo:nil repeats:YES];
        [[NSRunLoop mainRunLoop] addTimer:timer forMode:NSDefaultRunLoopMode];
    }
    
    self.liveStatus = ALIVC_LIVE_ROOM_STATUS_WATCHING;
    
    // 关闭自动锁屏
    [UIApplication sharedApplication].idleTimerDisabled = YES;
}

/**
 *  关闭player
 */
- (void)closeMediaPlayer {
    
    if(self.mediaPlayerCall != nil) {
        [self.mediaPlayerCall offlineChat];
        [self.mediaPlayerCall stopPlaying];
    }
    
    [self removeVideoChatObserver];
    
    self.mediaPlayerCall = nil;
    
    self.liveStatus = ALIVC_LIVE_ROOM_STATUS_NONE;
}

/**
 *  开始连麦
 */
- (void)createLiveCallWithPushUrl:(NSString *)pushUrl hostPlayUrl:(NSURL *)hostPlayUrl otherPlayUrls:(NSArray<NSURL *> *)otherPlayUrlArray otherPlayUids:(NSArray<NSString*> *)otherPlayUidArray
{
    // 开始连麦
    if (!self.mediaPlayerCall) {
        return;
    }
    
    [self.liveRoomView showLoadingViewWithMessage:@"启动连麦..."];
    
    int width = 180;
    int height = 320;
    
    self.publisherParam = [[NSMutableDictionary alloc] init];
    NSNumber* frontCamera = [[NSNumber alloc] initWithBool:YES];
    NSNumber* audioSampleRate = [[NSNumber alloc] initWithInt:32000];
    NSNumber* uploadTimeout = [[NSNumber alloc] initWithInt:5000];
    NSNumber* maxBitRate = [[NSNumber alloc] initWithInt:800];
    
    [self.publisherParam setObject:maxBitRate forKey:ALIVC_PUBLISHER_PARAM_MAXBITRATE];
    [self.publisherParam setObject:uploadTimeout forKey:ALIVC_PUBLISHER_PARAM_UPLOADTIMEOUT];
    [self.publisherParam setObject:audioSampleRate forKey:ALIVC_PUBLISHER_PARAM_AUDIOSAMPLERATE];
    [self.publisherParam setObject:frontCamera forKey:ALIVC_PUBLISHER_PARAM_FRONTCAMERAMIRROR];
    
    //设置推流最小码率
    NSNumber* minBitRate = [[NSNumber alloc] initWithInt:200];
    [self.publisherParam setObject:minBitRate forKey:ALIVC_PUBLISHER_PARAM_MINBITRATE];
    
    //设置推流初始码率
    NSNumber* originalBitRate = [[NSNumber alloc] initWithInt:400];
    [self.publisherParam setObject:originalBitRate forKey:ALIVC_PUBLISHER_PARAM_ORIGINALBITRATE];
    
    //设置是横屏推流还是竖屏推流
    NSNumber* landscape = [[NSNumber alloc] initWithInt:NO];
    [self.publisherParam setObject:landscape forKey:ALIVC_PUBLISHER_PARAM_LANDSCAPE];
    
    //设置推流是前置摄像头还是后置摄像头
    NSNumber* cameraPosition = [[NSNumber alloc] initWithInt:cameraPositionFront];
    [self.publisherParam setObject:cameraPosition forKey:ALIVC_PUBLISHER_PARAM_CAMERAPOSITION];
    
    [self.liveRoomView addSubview:self.liveRoomView.pushView]; 
    
    NSArray *playerViews = [self.liveRoomView addChatViewsWithArray:otherPlayUrlArray uidArrays:otherPlayUidArray];

    int ret = [self.mediaPlayerCall onlineChats:pushUrl width:width height:height preview:self.liveRoomView.pushView.chatView publisherParam:self.publisherParam hostPlayUrl:hostPlayUrl playerUrls:otherPlayUrlArray playerViews:playerViews];
    
    if (ret != 0) {
        [self.liveRoomView removeAllChatViews];
        [self.liveRoomView.pushView removeFromSuperview];
        [self.mediaPlayerCall offlineChat];
        [self showAlertViewWithMessage:[NSString stringWithFormat:@"连麦失败,ret=%d", ret]];
        [self.liveRoomView hideLoadingView];
        return;
    }
    
    NSLog(@"观众SDK开始连麦 %d", ret);
    self.playNotMixUrl = [hostPlayUrl absoluteString];
    [self.liveRoomView addSubview:self.liveRoomView.skinButton];
    [self.liveRoomView addSubview:self.liveRoomView.toggleButton];
    
    self.liveStatus = ALIVC_LIVE_ROOM_STATUS_CHATTING;
}

/**
 *  添加一个连麦
 */
- (void)addLiveCallWithPlayUrls:(NSArray<NSURL *> *)playUrlArray uids:(NSArray<NSString *> *) playUidArray
{
    NSArray *inviteViews = [self.liveRoomView addChatViewsWithArray:playUrlArray uidArrays:playUidArray];
    int ret = [self.mediaPlayerCall addChats:playUrlArray playerViews:inviteViews];
    if (ret != 0) {
        [self showAlertViewWithMessage:@"SDK添加连麦失败"];
    }
}

/**
 *  删除一个连麦
 */
- (void)removeLiveCallWithPlayUrls:(NSArray<NSString *> *)playUrlArray {
    
    NSMutableArray* urlArray = [[NSMutableArray alloc] init];
    for (NSString *playUrl in playUrlArray) {
        [urlArray addObject:[NSURL URLWithString:playUrl]];
        for (int i=0; i<[self.invitePlayUrlArray count]; i++) {
            LiveInviteInfo* info = [self.invitePlayUrlArray objectAtIndex:i];
            NSString* currentUrl = info.playUrl;
            if ([currentUrl isEqualToString:playUrl]) {
                [self.liveRoomView removeChatViewsWithUrl:playUrl];
                [self.invitePlayUrlArray removeObject:info];
                break;
            }
        }
    }
    
    int ret = [self.mediaPlayerCall removeChats:urlArray];
    if (ret != 0) {
        [self showAlertViewWithMessage:@"SDK移除连麦失败"];
    }
}

- (void)removeLiveCallWithPlayUids:(NSArray<NSString *> *)playUidArray {
    
    NSMutableArray* urlArray = [[NSMutableArray alloc] init];
    for (NSString *playUid in playUidArray) {
        for (int i=0; i<[self.invitePlayUrlArray count]; i++) {
            LiveInviteInfo* info = [self.invitePlayUrlArray objectAtIndex:i];
            NSString* uid = info.uid;
            if ([uid isEqualToString:playUid]) {
                [urlArray addObject:[NSURL URLWithString:info.playUrl]];
                [self.liveRoomView removeChatViewsWithUrl:info.playUrl];
                [self.invitePlayUrlArray removeObject:info];
                break;
            }
        }
    }
    
    int ret = [self.mediaPlayerCall removeChats:urlArray];
    if (ret != 0) {
        [self showAlertViewWithMessage:@"SDK移除连麦失败"];
    }
}

/**
 *  结束连麦
 */
- (void)closeLiveCall {
    if (!self.mediaPlayerCall) {
        return;
    }
    
    [self.liveRoomView showLoadingViewWithMessage:@"结束连麦..."];
    
    [self.liveRoomView.pushView removeFromSuperview];
    self.liveRoomView.mediaPalyerView.frame = [UIScreen mainScreen].bounds;
    [self.liveRoomView.skinButton removeFromSuperview];
    [self.liveRoomView.toggleButton removeFromSuperview];
    
    int ret = [self.mediaPlayerCall offlineChat];
    NSLog(@"观众SDK结束连麦 %d", ret);
    
    [self.liveRoomView.connectBtn setSelected:NO];

    self.liveStatus = ALIVC_LIVE_ROOM_STATUS_WATCHING;
    
    [self.invitePlayUrlArray removeAllObjects];
    
    [self.liveRoomView removeAllChatViews];
}

/**
 *  调试信息
 */
- (void) updateInfo {
    
    if(self.liveRoomView.textView != nil ){
        AlivcPublisherPerformanceInfo* info = [self.mediaPlayerCall getPublisherPerformanceInfo];
        
        NSMutableString *mutableString = [[NSMutableString alloc] init];
        int encode_speed = (info.videoEncodedBitrate+info.audioEncodedBitrate);
        int push_speed = (info.videoUploadedBitrate+info.audioUploadedBitrate);
        int video_queue_number = info.videoPacketsInBuffer;
        int audio_queue_number = info.audioPacketsInBuffer;
        [mutableString appendString:[NSString stringWithFormat:@"encode speed : %d kb/sec \n",encode_speed]];
        [mutableString appendString:[NSString stringWithFormat:@"push speed: %d kb/sec \n",push_speed]];
        [mutableString appendString:[NSString stringWithFormat:@"video frame number in queue : %d  \n",video_queue_number]];
        [mutableString appendString:[NSString stringWithFormat:@"audio frame number in queue : %d  \n",audio_queue_number]];
        [mutableString appendString:[NSString stringWithFormat:@"videoEncoderParamOfBitrate : %d \n",info.videoEncoderParamOfBitrate]];
        [mutableString appendString:[NSString stringWithFormat:@"currentlyUploadedVideoFramePts : %lld \n",info.currentlyUploadedVideoFramePts]];
        [mutableString appendString:[NSString stringWithFormat:@"currentlyUploadedAudioFramePts : %lld \n",info.currentlyUploadedAudioFramePts]];
        [mutableString appendString:[NSString stringWithFormat:@"previousKeyframePts : %lld \n",info.previousKeyframePts]];
        [mutableString appendString:[NSString stringWithFormat:@"totalFramesOfEncodedVideo : %lld \n",info.totalFramesOfEncodedVideo]];
        [mutableString appendString:[NSString stringWithFormat:@"totalTimeOfEncodedVideo : %lld \n",info.totalTimeOfEncodedVideo]];
        [mutableString appendString:[NSString stringWithFormat:@"totalTimeOfPublishing : %lld \n",info.totalTimeOfPublishing]];
        [mutableString appendString:[NSString stringWithFormat:@"totalFramesOfVideoUploaded : %lld \n",info.totalFramesOfVideoUploaded]];
        [mutableString appendString:[NSString stringWithFormat:@"totalSizeOfUploadedPackets : %lld \n",info.totalSizeOfUploadedPackets]];
        [mutableString appendString:[NSString stringWithFormat:@"dropDurationOfVideoFrames : %lld \n",info.dropDurationOfVideoFrames]];
        [mutableString appendString:[NSString stringWithFormat:@"videoDurationFromCaptureToUpload : %lld \n",info.videoDurationFromCaptureToUpload]];
        [mutableString appendString:[NSString stringWithFormat:@"audioDurationFromCaptureToUpload : %lld \n",info.audioDurationFromCaptureToUpload]];
        
        
        NSURL* url = [NSURL URLWithString: self.playUrl];
        if ([self.invitePlayUrlArray count]>0) {
            url = [NSURL URLWithString:[self.invitePlayUrlArray objectAtIndex:0].playUrl];
        }
        AlivcPlayerPerformanceInfo* playerInfo = [self.mediaPlayerCall getPlayerPerformanceInfo:url];
        if (playerInfo != nil) {
            [mutableString appendString:[NSString stringWithFormat:@"videoPacketsInBuffer : %d \n",playerInfo.videoPacketsInBuffer]];
            [mutableString appendString:[NSString stringWithFormat:@"audioPacketsInBuffer : %d \n",playerInfo.audioPacketsInBuffer]];
            [mutableString appendString:[NSString stringWithFormat:@"videoDurationFromDownloadToRender : %d \n",playerInfo.videoDurationFromDownloadToRender]];
            [mutableString appendString:[NSString stringWithFormat:@"audioDurationFromDownloadToRender : %d \n",playerInfo.audioDurationFromDownloadToRender]];
            [mutableString appendString:[NSString stringWithFormat:@"videoPtsOfLastPacketInBuffer : %lld \n",playerInfo.videoPtsOfLastPacketInBuffer]];
            [mutableString appendString:[NSString stringWithFormat:@"audioPtsOfLastPacketInBuffer : %lld \n",playerInfo.audioPtsOfLastPacketInBuffer]];
            [mutableString appendString:[NSString stringWithFormat:@"packetDownloadSpeed : %d \n",playerInfo.packetDownloadSpeed]];
        }
        
        self.liveRoomView.textView.text = mutableString;
    }
}

#pragma mark - 接收消息代理

//收到邀请连麦消息
- (void)onGetInviteMessage:(NSString *)inviterUid inviterName:(NSString*)inviterName inviterType:(int)inviterType
{
    // 判断状态
    if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_NONE || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
        return;
    }
    
    //弹出是否同意的对话框
    AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"收到连麦请求" icon:nil message:inviterUid delegate:self buttonTitles:@"拒绝连麦", @"同意连麦",nil];
    alert.tag = ALIVC_START_ALERT_TAG_VIDEOCALL_INVITE;
    [alert showInView:self.liveRoomView];
}

//主动发起连麦，收到对方同意连麦消息
- (void)onGetInviteAgreeMessage:(NSString*)inviteeUid inviteeName:(NSString*)inviteeName inviteeRoomId:(NSString*)inviteeRoomId inviterRoomId:(NSString*)inviterRoomId mainPlayUrl:(NSURL*)mainPlayUrl rtmpUrl:(NSString*)rtmpUrl otherPlayUrls:(NSArray *)otherPlayUrls otherPlayUids:(NSArray *)otherPlayUids
{
    self.mainUid = inviteeUid;
    [self createLiveCallWithPushUrl:rtmpUrl hostPlayUrl:mainPlayUrl otherPlayUrls:otherPlayUrls otherPlayUids:otherPlayUids];
    
    for (int i=0; i<[otherPlayUids count]; i++) {
        NSString* uid = [otherPlayUids objectAtIndex:i];
        NSString* url = [[otherPlayUrls objectAtIndex:i] absoluteString];
        LiveInviteInfo* info = [[LiveInviteInfo alloc] init];
        info.uid = uid;
        info.playUrl = url;
        [self.invitePlayUrlArray addObject:info];
    }
}

//不同意连麦
- (void)onGetInviteDisAgreeMessage:(NSString*)inviteeUid inviteeName:(NSString*)inviteeName
{
    if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
        [self showAlertViewWithMessage:@"对方不同意连麦"];
        self.liveStatus = ALIVC_LIVE_ROOM_STATUS_WATCHING;
    }
}

-(void) onGetInterruptLiveMessage:(NSString *)roomId uid:(NSString *)uid name:(NSString *)name
{
    [self.interruptUids addObject:uid];
    [self performSelector:@selector(delayMethod:) withObject:uid afterDelay:10.0];
}

-(void)delayMethod:(NSObject*)obj
{
    NSString* interruptId = (NSString*)obj;
    
    BOOL bFind = NO;
    for (NSString* uid in self.interruptUids) {
        if ([uid isEqualToString:interruptId]) {
            bFind = YES;
            break;
        }
    }
    
    if(bFind) {
        if ([self.userUid isEqualToString:interruptId] || [self.mainUid isEqualToString:interruptId]) {
            [self closeLiveCall];
        }
        else {
            [self removeLiveCallWithPlayUids:@[interruptId]];
        }
    }
}

//主播结束连麦
- (void)onGetHostCloseVideoCallMessage:(NSString*)roomId uid:(NSString*)uid name:(NSString*)name
{
    if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
        [self showAlertViewWithMessage:@"主播断开连麦"];
        [self closeLiveCall];
    }
}

//收到评论消息
- (void)onGetCommentMessage:(NSString*)uid name:(NSString*)name comment:(NSString*)comment
{
    if (self.chatDataArray.count >= 30) {
        [self.chatDataArray removeAllObjects];
    }
    
    //自己发送的消息不接收
    if([uid isEqualToString:self.userUid]) {
        return;
    }
    
    [self.chatDataArray addObject:[NSString stringWithFormat:@"%@:%@[%@]", name, comment, [self currentTime]]];
    [self.liveRoomView.chatTableView reloadData];
    
    // 滑动到底部
    [self.liveRoomView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatDataArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//点赞
- (void)onGetLikeMessage:(NSString*)uid name:(NSString*)name
{
    //自己发送的消息不接收
    if([uid isEqualToString:self.userUid]) {
        return;
    }
    
    AlivcLiveSpotView* spot = [[AlivcLiveSpotView alloc] initWithFrame:CGRectMake(0, 0, 36, 36)];
    [self.view addSubview:spot];
    CGPoint fountainSource = CGPointMake(kAlivcLiveScreenWidth - 100, kAlivcLiveScreenHeight - 36/2.0 - 10);
    spot.center = fountainSource;
    [spot animateInView:self.view];
    
    [self.chatDataArray addObject:[NSString stringWithFormat:@"点赞[%@]", [self currentTime]]];
    [self.liveRoomView.chatTableView reloadData];
    // 滑动到底部
    [self.liveRoomView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatDataArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//收到直播结束消息
- (void)onGetCloseLiveMessage:(NSString*)roomId
{
    AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"直播结束" delegate:self buttonTitles:@"重试", @"退出", nil];
    alert.tag = ALIVC_START_ALERT_TAG_ROOM_EXIT;
    [alert showInView:self.liveRoomView];
    return;
}

//混流失败
- (void)onGetMixFailureMessage:(NSString*)mainRoomId message:(NSString*)message
{
    //对于在连麦中的观众，则结束连麦
    if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
        AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"连麦混流失败" icon:nil message:message delegate:self buttonTitles:@"OK",nil];
        
        [alert showInView:self.liveRoomView];
        
        //todo:是否需要发送关闭连麦消息
        //[self sendLeaveVideoCallRequest];
        
        //调用结束连麦SDK
        [self closeLiveCall];
    }
}

//连麦混流状态
- (void)onGetMixStatusMessage:(NSString*)mainMixRoomId mixRoomId:(NSString*)mixRoomId message:(NSString*)message code:(NSString*)code
{
    if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
        //[self mixMessageClassifyWithMessage:code mainRoomId:mainMixRoomId];
    }
}

//推流开始消息
-(void)onGetStartLiveMessage:(NSString *)roomId uid:(NSString *)uid name:(NSString *)name playUrl:(NSString *)playUrl
{
    if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
        
        //观众自己的流不处理
        if ([self.userUid isEqualToString:uid]) {
            //观众自己的推流已经成功，可以结束连麦
            [self.liveRoomView.connectBtn setSelected:YES];
            return;
        }
        
        //如果自己未连麦，则不处理
        if (self.liveStatus != ALIVC_LIVE_ROOM_STATUS_CHATTING) {
            return;
        }
        
        //如果是主播中断后的推流 不处理
        if (self.playNotMixUrl != nil && [playUrl rangeOfString:self.playNotMixUrl].location != NSNotFound) {
            return;
        }
        
        //已经存在
        for (LiveInviteInfo* info in self.invitePlayUrlArray) {
            if ([info.playUrl isEqualToString:playUrl]) {
                return;
            }
        }
        
        
        NSURL* url = [NSURL URLWithString:playUrl];
        [self addLiveCallWithPlayUrls:@[url] uids:@[uid]];
        
        LiveInviteInfo* info = [[LiveInviteInfo alloc] init];
        info.uid = uid;
        info.playUrl = playUrl;
        [self.invitePlayUrlArray addObject:info];
    }
}

//加入连麦
- (void)onGetJoinVideoChatMessage:(LiveInviteInfo*)inviteInfo
{
    [self.chatDataArray addObject:[NSString stringWithFormat:@"%@:加入连麦[%@]", inviteInfo.name, [self currentTime]]];
    [self.liveRoomView.chatTableView reloadData];
    
    // 滑动到底部
    [self.liveRoomView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatDataArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//离开连麦
- (void)onGetLeaveVideoChatMessage:(LiveInviteInfo*)inviteInfo
{
    //如果自己收到离开消息，则是主播取消了你的连麦
    NSString *uid = inviteInfo.uid;
    if ([uid isEqualToString:self.userUid]){
        //断开推流
        [self closeLiveCall];
        return;
    }
    
    //如果正在连麦，则移除其他人的播放
    if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
        [self removeLiveCallWithPlayUrls:@[inviteInfo.playUrl]];
    }
    
    [self.chatDataArray addObject:[NSString stringWithFormat:@"%@:离开连麦[%@]", inviteInfo.name, [self currentTime]]];
    [self.liveRoomView.chatTableView reloadData];
    
    // 滑动到底部
    [self.liveRoomView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatDataArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//掉线情况
//0: 用户掉线
//1: server端掉线
- (void)onGetNetworkStatusMessage:(int)offlineType
{
//    if(offlineType == 0 || offlineType == 1) {
//        if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
//            [self sendLeaveVideoCallRequest];
//            [self closeLiveCall];
//        }
//    }
    
    [self sendRequestLiveChatRoomInfoWithTopicName:self.mnsModel.topic Tags:@[self.mnsModel.roomTag, self.mnsModel.userRoomTag]];
}

#pragma mark - 发送请求
/**
 *  发送加入直播请求
 */
- (void)sendJoinLiveRequest
{
    if (self.roomId.length == 0) {
        return;
    }
    
    [SendMessageManager watchLive:self.userUid roomId:self.roomId block:^(MNSInfoModel *mnsInfo, NSError *error) {
        
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"加入直播间error:%ld", (long)error.code]];
            return ;
        }
        
        self.mnsModel = mnsInfo;
        if (!self.mnsModel) {
            [self showAlertViewWithMessage:@"未请求到mnsModel,加入直播间失败"];
            return ;
        }
        [self sendRequestLiveChatRoomInfoWithTopicName:self.mnsModel.topic Tags:@[self.mnsModel.roomTag, self.mnsModel.userRoomTag]];
    }];
}

/**
 *  获取infoModel
 */
- (void)sendRequestLiveChatRoomInfoWithTopicName:(NSString *)topic Tags:(NSArray *)tags
{
    [SendMessageManager getMnsTopicInfo:topic tags:tags block:^(AlivcWebSocketInfoModel *infoModel, NSError *error) {
        
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"聊天SDK加入直播间失败error:%ld", (long)error.code]];
            return ;
        }
        
        [SendMessageManager joinChatRoomWithWebSocketInfoModel:infoModel success:^{
            NSLog(@"加入直播间成功");
            self.isNeedReconnectIM = NO;
        } error:^(NSError *error) { 
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"聊天SDK加入直播间失败error:%ld", (long)error.code]];
        }];
    }];
}

/**
 * 发送离开直播请求
 */
- (void)sendQuitLiveRoomRequest
{
    if (self.roomId.length == 0) {
        return;
    }
    
    [SendMessageManager leaveWatch:self.roomId uid:self.userUid block:^(NSError *error) {
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"离开直播间Request error:%ld", (long)error.code]];
            return ;
        }
    }];
}

/**
 *  发送连麦请求
 */
- (void)sendInviteVideoCallRequest
{
    NSArray* inviteeUids = [[NSArray alloc] initWithObjects:self.hostUid, nil];
    [SendMessageManager inviteVideoCall:self.roomId inviterUid:self.userUid inviteeUids:inviteeUids inviterType:1 block:^(NSError *error) {
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"发起连麦请求error:%ld", (long)error.code]];
            return ;
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            [self showAlertViewWithMessage:@"连麦请求发送成功"];
        });
        
        self.liveStatus = ALIVC_LIVE_ROOM_STATUS_CALLING;
    }];
}

/**
 *  获取连麦反馈
 */
- (void)sendInviteVideoCallFeedBack:(NSString*)inviterUid status:(int)status
{
    [SendMessageManager sendVideoCallFeedBack:inviterUid inviteeUid:self.userUid status:status inviterType:2 inviteeType:1 block:^(NSURL *mainPlayUrl, NSArray<NSURL *> *playUrls, NSArray<NSString*>* Uids, NSString *rtmpUrl, NSError *error) {
        
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"连麦反馈请求失败%ld", (long)error.code]];
            return ;
        }
        
        if (status == 1) {
            
            self.mainUid = inviterUid;
            
            //收到连麦邀请，如果同意，则开始推流
            [self createLiveCallWithPushUrl:rtmpUrl hostPlayUrl:mainPlayUrl otherPlayUrls:playUrls otherPlayUids:Uids];
            
            if (playUrls) {
                
                for (int i=0; i<[Uids count]; i++) {
                    NSString* uid = [Uids objectAtIndex:i];
                    NSString* url = [[playUrls objectAtIndex:i] absoluteString];
                    LiveInviteInfo* info  = [[LiveInviteInfo alloc] init];
                    info.uid = uid;
                    info.playUrl = url;
                    [self.invitePlayUrlArray addObject:info];
                }
            }
        }
    }];
}

/**
 *  发送离开连麦请求
 */
- (void)sendLeaveVideoCallRequest
{
    [SendMessageManager leaveVideoCall:self.roomId uid:self.userUid block:^(NSError *error) {
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"离开连麦请求失败error:%ld", (long)error.code]];
            return ;
        }
    }];
}

/**
 *  发送点赞请求
 */
- (void)sendSpotRequest
{
    [SendMessageManager likeWatch:self.userUid roomId:self.roomId block:^(NSError *error) {
        
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"点赞失败error:%ld", (long)error.code]];
        }
        
    }];
}

/**
 *  发送评论请求
 */
- (void)sendCommentRequestWithComment:(NSString *)comment
{
    [SendMessageManager sendLiveComments:self.roomId uid:self.userUid comments:comment block:nil];
}

// CDN推流连麦异常消息集中处理
- (void)mixMessageClassifyWithMessage:(NSString*)code mainRoomId:(NSString*)mainRoomId
{
    //  当前 是否是主流
    BOOL isMain = ![self.roomId isEqualToString:mainRoomId];
    
    NSString *remindMessage = @"";
    
    if ([code isEqualToString:@"Success"]) {
        
        // 混流成功
        self.isAnomalyTimeout = NO;
        return;
        
    } else if ([code isEqualToString:@"MainStreamNotExist"]) {
        
        // 主流不存在 混流失败
        if (isMain) {
            // 提示 当前用户断流
            remindMessage = @"您的推流暂时中断,正在恢复中...";
        } else {
            // 提示 对方断流
            remindMessage = @"对方推流暂时中断,正在恢复中...";
        }
        self.isAnomalyTimeout = YES;
        
    } else if ([code isEqualToString:@"MixStreamNotExist"]) {
        
        // 副流不存在 混流失败
        if (isMain) {
            // 提示 对方断流
            remindMessage = @"对方推流暂时中断,正在恢复中...";
        } else {
            // 提示 当前断流
            remindMessage = @"您的推流暂时中断,正在恢复中...";
        }
        self.isAnomalyTimeout = YES;
        
    } else if ([code isEqualToString:@"InternalError"]) {
        
        // 流内部错误 混流失败
        remindMessage = @"CDN处理内部错误InternalError,正在恢复中...";
        self.isAnomalyTimeout = YES;
        
    } else {
        // 无  未知错误
        remindMessage = @"连麦中断,未知错误,无对应错误码";
        self.isAnomalyTimeout = YES;
        
    }
    
    if (remindMessage.length > 0 && (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING)) {
        AlivcLiveAlertView *alertView = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:remindMessage delegate:nil buttonTitles:@"OK", nil];
        [alertView showInView:self.liveRoomView];
    }
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(20.0 * NSEC_PER_SEC)), dispatch_get_main_queue(),^{
        if (self.isAnomalyTimeout && (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING)) {
            AlivcLiveAlertView *alertView = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"等待重连超时，结束连麦" delegate:nil buttonTitles:@"OK", nil];
            [alertView showInView:self.liveRoomView];
            if (self.liveRoomView.connectBtn.isSelected) {
                [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
            }
        }
    });
    
}

#pragma mark - UITableViewDelegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.chatDataArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"chatTableViewID" forIndexPath:indexPath];
    
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.backgroundColor = [UIColor clearColor];
    
    UIView *bgView = [[UIView alloc] initWithFrame:cell.contentView.bounds];
    UILabel *label = [[UILabel alloc] initWithFrame:(CGRectMake(0, 2, CGRectGetWidth(cell.contentView.frame), CGRectGetHeight(cell.contentView.frame) - 4))];
    label.backgroundColor = [UIColor colorWithWhite:0 alpha:0.2];
    label.textColor = kAlivcColor;
    label.font = [UIFont fontWithName:@"Arial" size:14.0];
    [label.layer setMasksToBounds:YES];
    [label.layer setCornerRadius:12.0];
    label.text = self.chatDataArray[indexPath.row];
    [bgView addSubview:label];
    [cell setBackgroundView:bgView];

    return cell;
}

#pragma mark - AlivcLiveAlertViewDelegate
- (void)alertView:(AlivcLiveAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (alertView.tag == ALIVC_START_ALERT_TAG_VIDEOCALL_INVITE) {
        NSString* inviterUid = alertView.message;
        
        if (buttonIndex == 0) {
            // 不同意连麦
            [self sendInviteVideoCallFeedBack:inviterUid status:2];
        }
        if (buttonIndex == 1) {
            //同意
            [self sendInviteVideoCallFeedBack:inviterUid status:1];
        }
    }
    
    if (alertView.tag == ALIVC_START_ALERT_TAG_ROOM_EXIT) {
        if (buttonIndex == 0) {
            if (self.mediaPlayerCall && self.playUrl) {
                [self.mediaPlayerCall reconnect:[NSURL URLWithString:self.playUrl]];
            }
        }
        if (buttonIndex == 1) {
            
            //关闭连麦
            [self closeLiveCall];
            
            //通知服务器退出直播间
            [self sendQuitLiveRoomRequest];
            
            //关闭播放器
            [self closeMediaPlayer];
            
            // 退出直播间
            [SendMessageManager quitChatRoomSuccess:^{
                NSLog(@"退出直播间成功");
            } error:^(NSError *error) {
            }];
            
            [self dismissViewControllerAnimated:YES completion:nil];
            [UIApplication sharedApplication].idleTimerDisabled = NO;
        }
    }
    
    if (alertView.tag == ALIVC_START_ALERT_TAG_VIDEOCALL_EXIT) {
        if (buttonIndex == 1) {
            if (self.liveRoomView.connectBtn.selected) {
                [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
            }
        }
    }
}

#pragma mark - ButtonAction

- (void)addButtonActions
{
    [self.liveRoomView.doneBtn addTarget:self action:@selector(DonePressed:) forControlEvents:UIControlEventTouchUpInside];
    [self.liveRoomView.connectBtn addTarget:self action:@selector(startOrStopVideoCallBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.liveRoomView.spotButton addTarget:self action:@selector(spotButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.liveRoomView.skinButton addTarget:self action:@selector(skinButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.liveRoomView.toggleButton addTarget:self action:@selector(toggleButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.liveRoomView.chatSendButton addTarget:self action:@selector(chatSendButtonAction:) forControlEvents:(UIControlEventTouchUpInside)];
}

- (void)DonePressed:(UIButton *)button
{
    // 如果有连麦，关闭连麦
    if (self.liveRoomView.connectBtn.selected) {
        AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"当前正在与主播连麦，请先结束连麦在退出观看" delegate:self buttonTitles:@"取消", @"结束连麦", nil];
        alert.tag = ALIVC_START_ALERT_TAG_VIDEOCALL_EXIT;
        [alert showInView:self.liveRoomView];
        return;
    }
    
    //通知服务器退出直播间
    [self sendQuitLiveRoomRequest];
    
    //关闭播放器
    [self closeMediaPlayer];
    
    // 退出直播间
    [SendMessageManager quitChatRoomSuccess:^{
        NSLog(@"退出直播间成功");
    } error:^(NSError *error) {
    }];
    
    [self dismissViewControllerAnimated:YES completion:nil];
    [UIApplication sharedApplication].idleTimerDisabled = NO;
}

- (void)startOrStopVideoCallBtnAction:(UIButton *)button
{
    if (button.selected) {
        //发送结束连麦请求
        [self sendLeaveVideoCallRequest];
        
        // 断开连麦
        [self closeLiveCall];
        
    } else {
        
        //已经正在连麦中，或者请求连麦中，反正重复申请
        if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CALLING || self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
            return;
        }
        
        // 发送开始连麦请求
        [self sendInviteVideoCallRequest];
    }
}

//发送聊天消息
- (void)chatSendButtonAction:(UIButton *)sender
{
    [self.liveRoomView.chattextField resignFirstResponder];
    
    if (self.liveRoomView.chattextField.text.length == 0) {
        self.liveRoomView.chattextField.placeholder = @"发送的弹幕不能为空";
    } else {
        
        [self.chatDataArray addObject:[NSString stringWithFormat:@"发送-%@[%@]",self.liveRoomView.chattextField.text, [self currentTime]]];
        [self.liveRoomView.chatTableView reloadData];
        // 滑动到底部
        [self.liveRoomView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatDataArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
        
        
        [self sendCommentRequestWithComment:self.liveRoomView.chattextField.text];
        
        self.liveRoomView.chattextField.text = @"";
        self.liveRoomView.chattextField.placeholder = @"发送弹幕";
    }
}

//发送点赞消息
- (void)spotButtonAction:(UIButton *)sender
{
    AlivcLiveSpotView* spot = [[AlivcLiveSpotView alloc] initWithFrame:CGRectMake(0, 0, 36, 36)];
    [self.view addSubview:spot];
    CGPoint fountainSource = CGPointMake(kAlivcLiveScreenWidth - 100, kAlivcLiveScreenHeight - 36/2.0 - 10);
    spot.center = fountainSource;
    [spot animateInView:self.view];
    
    [self.chatDataArray addObject:[NSString stringWithFormat:@"发送-点赞[%@]", [self currentTime]]];
    [self.liveRoomView.chatTableView reloadData];
    // 滑动到底部
    [self.liveRoomView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatDataArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
    
    [self sendSpotRequest];
}

//打开美颜
- (void)skinButtonAction:(UIButton *)sender
{
    NSNumber* number = [[NSNumber alloc] initWithBool:sender.selected];
    NSString* key = ALIVC_FILTER_PARAM_BEAUTY_ON;
    NSDictionary* dic = [[NSDictionary alloc] initWithObjectsAndKeys:number, key,nil];
    [self.mediaPlayerCall setFilterParam:dic];

    [sender setSelected:!sender.selected];
}

//切换前后摄像头
- (void)toggleButtonAction:(UIButton *)sender
{
    [self.mediaPlayerCall switchCamera];
}

//去掉键盘输入
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    [self.liveRoomView.chattextField resignFirstResponder];
}

#pragma mark - NSNotification
- (void) addVideoChatObserver
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnConnectError:)
                                                 name:AlivcVideoChatPublisherOpenFailed object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnNetworkTimeout:)
                                                 name:AlivcVideoChatPublisherSendDataTimeout object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoCaptureError:)
                                                 name:AlivcVideoChatPublisherVideoCaptureDisabled object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnAudioCaptureDisabled:)
                                                 name:AlivcVideoChatPublisherAudioCaptureDisabled object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoInitError:)
                                                 name:AlivcVideoChatPublisherVideoEncoderInitFailed object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnAudioInitError:)
                                                 name:AlivcVideoChatPublisherAudioEncoderInitFailed object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnMemoryMallocError:)
                                                 name:AlivcVideoChatMemoryPool object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnNetworkSlow:)
                                                 name:AlivcVideoChatPublisherNetSpeedPoor object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnAudioEncodeError:)
                                                 name:AlivcVideoChatPublisherEncodeAudioFailed object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoEncodeError:)
                                                 name:AlivcVideoChatPublisherEncodeVideoFailed object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoChatStartBuffering:)
                                                 name:AlivcVideoChatPlayerStartBuffering object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoChatEndBuffering:)
                                                 name:AlivcVideoChatPlayerEndBuffering object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onChatParterDisplayFirstFrame:)
                                                 name:AlivcVideoChatPlayerFirstFrameRender object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPlayerReadPacketTimeout:)
                                                 name:AlivcVideoChatPlayerReadPacketTimeout object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPlayerNoDisplayViewer:)
                                                 name:AlivcVideoChatPlayerNoDisplayViewer object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPlayerInvalidCodec:)
                                                 name:AlivcVideoChatPlayerInvalidCodec object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPlayerOpenFailed:)
                                                 name:AlivcVideoChatPlayerOpenFailed object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onAudioCaptureError:) name:AlivcVideoChatPublisherAudioCaptureError object:self.mediaPlayerCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onCaptureFpsSlow:) name:AlivcVideoChatPublisherCaptureFpsSlow object:self.mediaPlayerCall];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(resignActive)
                                                 name:UIApplicationDidEnterBackgroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(becomeActive)
                                                 name:UIApplicationWillEnterForegroundNotification object:nil];
}

-(void)removeVideoChatObserver
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)OnConnectError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"连接断开" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
    
    if (self.liveStatus == ALIVC_LIVE_ROOM_STATUS_CHATTING) {
        [self sendLeaveVideoCallRequest];
        [self closeLiveCall];
    }
}

- (void)OnNetworkTimeout:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"推流超时" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
    
    if (self.liveRoomView.connectBtn.selected) {
        [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
    }
}

- (void)OnVideoCaptureError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"视频采集被禁止" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
}

- (void)OnAudioCaptureDisabled:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"音频采集被禁止" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
}

- (void)OnVideoInitError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"视频初始化失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.liveRoomView.connectBtn.selected){
        [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
    }
}

- (void)OnAudioInitError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"音频初始化失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.liveRoomView.connectBtn.selected) {
        [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
    }
}

- (void)OnMemoryMallocError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"内存分配失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    [self.liveRoomView hideLoadingView];
    if (self.liveRoomView.connectBtn.selected) {
        [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
    }
}

- (void)OnNetworkSlow:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"网速太慢,直播关闭" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];

}

- (void)OnAudioEncodeError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"音频编码失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.liveRoomView.connectBtn.selected) {
        [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
    }
}

- (void)OnVideoEncodeError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"视频编码失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.liveRoomView.connectBtn.selected) {
        [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
    }
}

- (void)onChatParterDisplayFirstFrame:(NSNotification *)notification
{
    UInt64 currentTime = [[NSDate date] timeIntervalSince1970] * 1000;
    NSLog(@"%llu", currentTime);
    NSLog(@"%llu", self.createPlayTime);
    NSLog(@"首帧时间:%llu", currentTime - self.createPlayTime);
    self.liveRoomView.timeLabel.text = [NSString stringWithFormat:@"首帧:%0.3llu", currentTime - self.createPlayTime];
    
    self.liveRoomView.doneBtn.userInteractionEnabled = YES;
    self.liveRoomView.connectBtn.userInteractionEnabled = YES;
    self.liveRoomView.spotButton.userInteractionEnabled = YES;
    [self.liveRoomView hideLoadingView];
}

- (void)OnVideoChatStartBuffering:(NSNotification *)notification
{
    NSLog(@"video chat start to buffering\n");
}

- (void)OnVideoChatEndBuffering:(NSNotification *)notification
{
    NSLog(@"video chat end buffering\n");
}

- (void)OnPlayerReadPacketTimeout:(NSNotification *)notification
{
    NSString* playUrl = nil;
    NSDictionary* diction = [notification userInfo];
    if (diction) {
        playUrl = [diction objectForKey:@"playUrl"];
    }
    
    if (playUrl == nil) {
        assert(0);
    }
    
    [self.liveRoomView hideLoadingView];
    [self.mediaPlayerCall reconnect:[NSURL URLWithString:playUrl]];
}

- (void)OnPlayerNoDisplayViewer:(NSNotification *)notification
{
    NSString* playUrl = nil;
    NSDictionary* diction = [notification userInfo];
    if (diction) {
        playUrl = [diction objectForKey:@"playUrl"];
    }
    
    if (playUrl == nil) {
        assert(0);
    }
    
    [self.liveRoomView hideLoadingView];
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:playUrl message:@"播放无显示窗口" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}

- (void)OnPlayerInvalidCodec:(NSNotification *)notification
{
    NSString* playUrl = nil;
    NSDictionary* diction = [notification userInfo];
    if (diction) {
        playUrl = [diction objectForKey:@"playUrl"];
    }
    
    if (playUrl == nil) {
        assert(0);
    }
    
    [self.liveRoomView hideLoadingView];
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:playUrl message:@"无效的解码器" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}

- (void)onAudioCaptureError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"错误" message:@"音频采集错误,结束连麦" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
    
    //关闭连麦
    if (self.liveRoomView.connectBtn.selected) {
        [self startOrStopVideoCallBtnAction:self.liveRoomView.connectBtn];
    }
}

- (void)onCaptureFpsSlow:(NSNotification *)notification
{
    NSLog(@"视频采集帧率低");
}

- (void)OnPlayerOpenFailed:(NSNotification *)notification
{
    NSString* playUrl = nil;
    NSDictionary* diction = [notification userInfo];
    if (diction) {
        playUrl = [diction objectForKey:@"playUrl"];
    }
    if (playUrl == nil) {
        assert(0);
    }
    //if(self.liveStatus == ALIVC_LIVE_ROOM_STATUS_WATCHING)
    {
        AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:playUrl icon:nil message:@"打开失败，离开房间" delegate:self buttonTitles:@"重试",@"确定", nil];
        alert.tag = ALIVC_START_ALERT_TAG_ROOM_EXIT;
        [alert showInView:self.liveRoomView];
    }
    [self.liveRoomView hideLoadingView];
}

- (void)resignActive
{
    [self.mediaPlayerCall pause];
}

- (void)becomeActive
{
    [self.mediaPlayerCall resume];
}

- (NSString *)currentTime
{
    NSDate * date = [NSDate date];
    NSTimeInterval sec = [date timeIntervalSinceNow];
    NSDate * currentDate = [[NSDate alloc] initWithTimeIntervalSinceNow:sec];
    
    NSDateFormatter * df = [[NSDateFormatter alloc] init];
    [df setDateFormat:@"HH:mm:ss"];
    
    return [df stringFromDate:currentDate];
}

#pragma mark - AlertView
- (void)showAlertViewWithMessage:(NSString *)message
{
    AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:message delegate:self buttonTitles:@"OK", nil];
    [alert showInView:self.liveRoomView];
}


@end
