//
//  LoginViewController.m
//  DemoAlivcLive
//
//  Created by LYZ on 16/8/8.
//  Copyright © 2016年 Alibaba. All rights reserved.
//

#import "LoginViewController.h"
#import "AFNetworking.h"
#import "MBProgressHUD.h"
#import "LoginView.h"
#import "AlivcLiveAlertView.h"
#import "AlivcRequestManager.h"
#import "SendMessageManager.h"

@interface LoginViewController ()<UITextFieldDelegate, UIAlertViewDelegate>

@property (nonatomic, strong) LoginView *loginView;

@property (nonatomic, assign) BOOL isNetWorking;

@end

@implementation LoginViewController

- (void)loadView {
    
    self.loginView = [[LoginView alloc] initWithFrame:([UIScreen mainScreen].bounds)];
    self.view = self.loginView;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.navigationItem.title = @"登录";
    
    // Do any additional setup after loading the view from its nib.
    [self.loginView.backButton addTarget:self action:@selector(backButtonAction:) forControlEvents:(UIControlEventTouchUpInside)];
    [self.loginView.nextButton addTarget:self action:@selector(nextbuttonAction:) forControlEvents:(UIControlEventTouchUpInside)];
    
    [self checkHost];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    self.isNetWorking = NO;
    // 检测网络
    AFNetworkReachabilityManager *manager = [AFNetworkReachabilityManager sharedManager];
    [manager startMonitoring];
    [manager setReachabilityStatusChangeBlock:^(AFNetworkReachabilityStatus status) {
        
        switch (status) {
            case AFNetworkReachabilityStatusUnknown:
                self.isNetWorking = NO;
                break;
            case AFNetworkReachabilityStatusNotReachable:
                self.isNetWorking = NO;
                break;
            case AFNetworkReachabilityStatusReachableViaWWAN:
                self.isNetWorking = YES;
                break;
            case AFNetworkReachabilityStatusReachableViaWiFi:
                self.isNetWorking = YES;
                break;
            default:
                break;
        }
    }];

}

- (BOOL)checkHost {
    // check Host
    NSString *host = kAlivcLiveHost;
    if (!host.length) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"Host为空，请先在PrefixHeader.h中修改kAlivcLiveHost" delegate:nil cancelButtonTitle:@"确定" otherButtonTitles: nil];
        [alert show];
        return NO;
    }
    return YES;
}

#pragma mark - Login
- (void)login {
    
    if (![self checkHost]) {
        return;
    }
    
    if (self.loginView.nameTextField.text.length == 0) {
        
        
        AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"请输入用户名" delegate:nil buttonTitles:@"OK",nil];
        [alert showInView:self.loginView];
        self.loginView.nextButton.userInteractionEnabled = YES;
        return;
    }
    
    if (!self.isNetWorking) {
        AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"无网络，请检查WIFI或者移动网络是否打开" delegate:nil buttonTitles:@"OK",nil];
        [alert showInView:self.loginView];
        self.loginView.nextButton.userInteractionEnabled = YES;
        return;
    }
    
    MBProgressHUD *mbHub = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    mbHub.label.text = @"登录中...";
    
    [SendMessageManager userLogIn:self.loginView.nameTextField.text password:self.loginView.passwordTextField.text block:^(UserInfo *userInfo, NSError *error) {
      
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [mbHub hideAnimated:YES];
                AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:[NSString stringWithFormat:@"登录失败error:%ld", (long)error.code] delegate:nil buttonTitles:@"OK",nil];
                [alert showInView:self.loginView];
                [self.loginView.nextButton setSelected:NO];
                self.loginView.nextButton.userInteractionEnabled = YES;
            });
            return ;
        }
        
        NSString *uid = userInfo.id;
        NSString *name = userInfo.name;
        dispatch_async(dispatch_get_main_queue(), ^{
            [mbHub hideAnimated:YES];
            [self.delegate LoginViewControllerSendBackUid:uid name:name];
            [self dismissViewControllerAnimated:YES completion:nil];
        });
        
    }];
}

#pragma mark - Action
- (void)backButtonAction:(UIButton *)sender {
    
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)nextbuttonAction:(UIButton *)sender {
    
    [self.loginView.nameTextField resignFirstResponder];
    [self.loginView.passwordTextField resignFirstResponder];
    
    sender.userInteractionEnabled = NO;
    
    [sender setSelected:!sender.selected];
    
    [self login];
    
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    
    [self.loginView.nameTextField resignFirstResponder];
    [self.loginView.passwordTextField resignFirstResponder];
}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    
    [self.loginView.nextButton setSelected:NO];
    self.loginView.nextButton.userInteractionEnabled = YES;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
