//
//  StartLiveViewController.m
//  DemoAlivcLive
//
//  Created by LYZ on 16/8/2.
//  Copyright © 2016年 Alibaba. All rights reserved.
//

#import "StartLiveViewController.h"
#import <AlivcVideoChat/AlivcVideoChat.h>
#import "AFNetworking.h"
#import "AlivcLiveSpotView.h"
#import "MBProgressHUD.h"
#import "StartLiveView.h"
#import "AlivcLiveAlertView.h"
#import "MNSInfoModel.h"
#import "LiveInviteInfo.h"
#import "SendMessageManager.h"
#import "RecieveMessageManager.h"
#import "ALiViewTableViewCell.h"


typedef NS_ENUM(NSInteger, ALIVC_START_LIVE_STATUS) {
    ALIVC_START_LIVE_STATUS_NONE = 0,               // 无
    ALIVC_START_LIVE_STATUS_LIVING = 1,             // 直播中 (无人连麦)
    ALIVC_START_LIVE_STATUS_CALLING = 2,            // 连麦连接中
    ALIVC_START_LIVE_STATUS_CHATTING = 3,           // 连麦中 1名
    ALIVC_START_LIVE_STATUS_CHATTING_CLOSING = 4,   // 结束连麦中
};


@interface StartLiveViewController ()<AlivcLiveAlertViewDelegate, UITableViewDelegate, UITableViewDataSource,StartLiveViewDelegate,RecieveMessageDelegate,ALiCustomListCellModelDelegate>

// UI
@property (nonatomic, strong) StartLiveView *startLiveView;
@property (nonatomic, strong) AlivcLiveAlertView *inviteAlertView;

// 连麦SDK相关
@property (nonatomic, strong) NSMutableDictionary *playerParam;
@property (nonatomic, strong) NSMutableDictionary *publisherParam;
@property (nonatomic, strong) AlivcVideoChatHost *publiserVideoCall;

// 直播间聊天室SDK相关
@property (nonatomic, strong) MNSInfoModel *mnsModel;

//接收消息管理
@property (nonatomic, strong) RecieveMessageManager *mRecieveManager;

//连麦管理
@property (nonatomic, assign) ALIVC_START_LIVE_STATUS liveStatus;
@property (nonatomic, strong) NSMutableArray<LiveInviteInfo *> *currentInviterArray; // 当前连麦的全部uid
@property (nonatomic, strong) NSString *rtmpURLString;
@property (nonatomic, strong) NSString *roomId;

@property (nonatomic, strong) NSMutableArray *chatArray; // 聊天消息array
@property (nonatomic, strong) NSMutableArray *inviterAudienceList; // 主播同意列表
@property (nonatomic, strong) NSMutableArray<UserInfo*> *audienceListArray;// 直播间观众list
@property (nonatomic, strong) NSMutableArray *videoCallAudience;// 当前正在连麦的观众list
@property (nonatomic, assign) BOOL isNeedReconnectIM; // 是否重连IM
@property (nonatomic, strong) NSMutableArray *interruptUids; // 主播同意列表

@end

@implementation StartLiveViewController

- (void)loadView {
    
    self.startLiveView = [[StartLiveView alloc] initWithFrame:([UIScreen mainScreen].bounds)];
    [self.startLiveView setTableViewDataSourceDelegate:self];
    self.startLiveView.delegate = self;
    self.view = self.startLiveView;
}

-(void) initVariable
{
    self.mRecieveManager = [[RecieveMessageManager alloc] init];
    self.mRecieveManager.delegate = self;
    
    self.audienceListArray = [NSMutableArray array];
    self.chatArray = [NSMutableArray array];
    self.inviterAudienceList = [NSMutableArray array];
    
    self.currentInviterArray = [NSMutableArray array];
    self.interruptUids = [NSMutableArray array];
    self.liveStatus = ALIVC_START_LIVE_STATUS_NONE;
    
    self.inviteAlertView = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"连麦中..." delegate:nil buttonTitles: nil];
    [self addButtonActions];
    
    self.startLiveView.nameLabel.text = [NSString stringWithFormat:@"%@(%@)", self.liveName, self.uid];
    
    [self createPublisher];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self initVariable];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    // 检测网络
    AFNetworkReachabilityManager *manager = [AFNetworkReachabilityManager sharedManager];
    [manager startMonitoring];
    [manager setReachabilityStatusChangeBlock:^(AFNetworkReachabilityStatus status) {
        
        switch (status) {
            case AFNetworkReachabilityStatusUnknown:
            {
                [self showAlertViewWithMessage:@"无网络，请检查WIFI或者移动网络是否打开"];
            }
                break;
            case AFNetworkReachabilityStatusNotReachable:
            {
                [self showAlertViewWithMessage:@"无网络，请检查WIFI或者移动网络是否打开"];
            }
                break;
            case AFNetworkReachabilityStatusReachableViaWWAN:
            {
                NSLog(@"3G|4G");
                
                // IM重连
                if (self.isNeedReconnectIM) {
                    [self sendRequestLiveChatRoomInfoWithTopicName:self.mnsModel.topic Tags:@[self.mnsModel.roomTag, self.mnsModel.userRoomTag]];
                }
                
            }
                break;
            case AFNetworkReachabilityStatusReachableViaWiFi:
            {
                NSLog(@"WiFi");
                
                // IM重连
                if (self.isNeedReconnectIM) {
                    [self sendRequestLiveChatRoomInfoWithTopicName:self.mnsModel.topic Tags:@[self.mnsModel.roomTag, self.mnsModel.userRoomTag]];
                }
            }
                break;
            default:
                break;
        }
    }];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - 接收消息代理

//收到连麦邀请消息
- (void)onGetInviteMessage:(NSString *)inviterUid inviterName:(NSString*)inviterName inviterType:(int)inviterType
{
    //todo: 未判断已经连麦的人数，超过3人则返回
    
    if (inviterType == 2) {
        NSLog(@"主播不能和主播进行连麦");
        return;
    }
    
    for (LiveInviteInfo* info in self.currentInviterArray) {
        if ([info.uid isEqualToString:inviterUid]) {
            NSLog(@"改用户已经在连麦中");
            return;
        }
    }
    
    self.liveStatus = ALIVC_START_LIVE_STATUS_CALLING;
    self.inviteAlertView.message = [NSString stringWithFormat:@"%@邀请连麦,接入中",inviterName];
    
    [self.inviterAudienceList addObject:inviterUid];
    
    //弹出是否同意的对话框
    AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"收到连麦请求" icon:nil message:inviterUid delegate:self buttonTitles:@"拒绝连麦", @"同意连麦",nil];
    alert.tag = ALIVC_START_ALERT_TAG_VIDEOCALL_INVITE;
    [alert showInView:self.startLiveView];
}

//推流消息
- (void)onGetStartLiveMessage:(NSString *)roomId uid:(NSString *)uid name:(NSString *)name playUrl:(NSString *)playUrl
{
    BOOL bFind = NO;
    for (NSString* interruptId in self.interruptUids) {
        if ([interruptId isEqualToString:uid]) {
            bFind = YES;
            [self.interruptUids removeObject:interruptId];
            break;
        }
    }
    
    if ([self.uid isEqualToString:uid] == YES) {
        return;
    }
    
    bFind = NO;
    for (NSString* strUid in self.inviterAudienceList) {
        if ([strUid isEqualToString:uid]) {
            bFind = YES;
            break;
        }
    }
    
    if (bFind == NO){
        [self showAlertViewWithMessage:@"提示：该推流的用户不在列表中"];
        return;
    }
    
    //对方已经推流，可以打开对方地址进行播放
    NSURL* url = [NSURL URLWithString:playUrl];
    
    //如果已经存在，则返回，对方有可能重复推流
    for (LiveInviteInfo* info in self.currentInviterArray) {
        if ([info.playUrl isEqualToString:playUrl]) {
            return;
        }
    }
    
    if ([self.currentInviterArray count] == 0) {
        [self createLiveCallWithInviteePlayUrls:@[url] uids:@[uid]];
    }
    else {
        [self addLiveCallWithPlayUrls:@[url] uids:@[uid]];
    }
    
    bFind = NO;
    for (LiveInviteInfo* info in self.currentInviterArray) {
        if ([info.uid isEqualToString:uid]) {
            info.name = name;
            info.playUrl = playUrl;
            info.roomId = roomId;
            bFind = YES;
            break;
        }
    }
    
    if (bFind == NO) {
        LiveInviteInfo* info = [[LiveInviteInfo alloc] init];
        info.uid = uid;
        info.name = name;
        info.playUrl = playUrl;
        info.roomId = roomId;
        
        [self.currentInviterArray addObject:info];
    }
    
    [self.startLiveView.connectBtn setSelected:YES];
}

//同意连麦
- (void)onGetInviteAgreeMessage:(NSString*)inviteeUid inviteeName:(NSString*)inviteeName inviteeRoomId:(NSString*)inviteeRoomId inviterRoomId:(NSString*)inviterRoomId mainPlayUrl:(NSString*)mainPlayUrl
{
    [self.chatArray addObject:[NSString stringWithFormat:@"%@同意连麦", inviteeName]];
    [self.startLiveView.chatTableView reloadData];
    
    [self.inviterAudienceList addObject:inviteeUid];
    
    // 滑动到底部
    [self.startLiveView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//不同意连麦
- (void)onGetInviteDisAgreeMessage:(NSString*)inviteeUid inviteeName:(NSString*)inviteeName
{
    [self.chatArray addObject:[NSString stringWithFormat:@"%@不同意连麦", inviteeName]];
    [self.startLiveView.chatTableView reloadData];
    
    //移除
    [self.inviterAudienceList removeObject:inviteeUid];
    for (LiveInviteInfo* info in self.currentInviterArray) {
        if([info.uid isEqualToString:inviteeUid]){
            [self.currentInviterArray removeObject:info];
            break;
        }
    }
    
    // 滑动到底部
    [self.startLiveView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//主播结束连麦
- (void)onGetHostCloseVideoCallMessage:(NSString*)roomId uid:(NSString*)uid name:(NSString*)name
{
    //如果正在连麦，则结束连麦
    if (self.liveStatus == ALIVC_START_LIVE_STATUS_CALLING || self.liveStatus == ALIVC_START_LIVE_STATUS_CHATTING) {
        [self closeLiveCall];
    }
}

//评论
- (void)onGetCommentMessage:(NSString*)uid name:(NSString*)name comment:(NSString*)comment
{
    //自己不接收自己的消息
    if ([self.uid isEqualToString:uid])
        return;
    
    [self.chatArray addObject:[NSString stringWithFormat:@"%@:%@[%@]", name, comment, [self currentTime]]];
    [self.startLiveView.chatTableView reloadData];
    
    // 滑动到底部
    [self.startLiveView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//点赞
- (void)onGetLikeMessage:(NSString*)uid name:(NSString*)name
{
    //自己不接收自己的消息
    if ([self.uid isEqualToString:uid])
        return;
    
    [self.chatArray addObject:[NSString stringWithFormat:@"%@:点赞[%@]", name, [self currentTime]]];
    [self.startLiveView.chatTableView reloadData];
    
    // 滑动到底部
    [self.startLiveView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
    AlivcLiveSpotView *spot = [[AlivcLiveSpotView alloc] initWithFrame:CGRectMake(0, 0, 36, 36)];
    [self.view addSubview:spot];
    CGPoint fountainSource = CGPointMake(kAlivcLiveScreenWidth - 100, kAlivcLiveScreenHeight - 36/2.0 - 10);
    spot.center = fountainSource;
    [spot animateInView:self.view];
}

//加入直播间
- (void) onGetInRoomMessage:(NSString *)name
{
    [self.chatArray addObject:[NSString stringWithFormat:@"%@加入直播间", name]];
    [self.startLiveView.chatTableView reloadData];
    // 滑动到底部
    [self.startLiveView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//离开直播间
- (void) onGetOutRoomMessage:(NSString *)name
{
    [self.chatArray addObject:[NSString stringWithFormat:@"%@离开了直播间", name]];
    [self.startLiveView.chatTableView reloadData];
    // 滑动到底部
    [self.startLiveView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//直播结束
- (void)onGetCloseLiveMessage:(NSString*)roomId
{
    AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"直播间结束,请重新创建直播" delegate:self buttonTitles:@"退出", nil];
    [alert showInView:self.startLiveView];
}

//混流失败
- (void)onGetMixFailureMessage:(NSString*)mainRoomId
{
    if (self.liveStatus == ALIVC_START_LIVE_STATUS_CHATTING || self.liveStatus == ALIVC_START_LIVE_STATUS_CALLING) {
        
        self.inviteAlertView.message = @"连麦失败";
        [self.inviteAlertView performSelector:@selector(hide) withObject:self.inviteAlertView afterDelay:2.0f];
        
        //混流失败，结束连麦
        [self closeLiveCall];
    }
}

//连麦混流状态
- (void)onGetMixStatusMessage:(NSString*)mainMixRoomId mixRoomId:(NSString*)mixRoomId message:(NSString*)message code:(NSString*)code
{
    if (self.liveStatus == ALIVC_START_LIVE_STATUS_CHATTING || self.liveStatus == ALIVC_START_LIVE_STATUS_CALLING) {
        //[self mixMessageClassifyWithMessage:code mainRoomId:mainMixRoomId];
    }
}

//主播收到加入连麦消息
- (void)onGetJoinVideoChatMessage:(LiveInviteInfo*)inviteInfo
{
    if(self.roomId == inviteInfo.roomId) {
        [self.chatArray addObject:[NSString stringWithFormat:@"%@加入连麦",inviteInfo.name]];
        [self.startLiveView.chatTableView reloadData];
        // 滑动到底部
        [self.startLiveView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
    }
}

//离开连麦
- (void)onGetLeaveVideoChatMessage:(LiveInviteInfo*)inviteInfo
{
    //观众主动断开
    //[self removeLiveCallWithPlayurl:inviteInfo.playUrl];
    [self removeLiveCallWithPlayUid:inviteInfo.uid];
    
    //移除最后一个连麦，则整个连麦关闭
    if ([self.currentInviterArray count] == 0) {
        [self.startLiveView.connectBtn setSelected:NO];
        [self.startLiveView removeAllChatViews];
        if (self.publiserVideoCall) {
            int ret = [self.publiserVideoCall abortChat];
            NSLog(@"结束连麦:%d", ret);
        }
        self.liveStatus = ALIVC_START_LIVE_STATUS_LIVING;
        return;
    }
    
    [self.chatArray addObject:[NSString stringWithFormat:@"%@离开连麦",inviteInfo.name]];
    [self.startLiveView.chatTableView reloadData];
    // 滑动到底部
    [self.startLiveView.chatTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:(self.chatArray.count - 1) inSection:0] atScrollPosition:(UITableViewScrollPositionBottom) animated:YES];
}

//断流
-(void)onGetInterruptLiveMessage:(NSString *)roomId uid:(NSString *)uid name:(NSString *)name
{
    [self.interruptUids addObject:uid];
    [self performSelector:@selector(delayMethod:) withObject:uid afterDelay:10.0];
}

-(void)delayMethod:(NSObject*)obj
{
    NSString* interruptId = (NSString*)obj;
    
    BOOL bFind = NO;
    for (NSString* uid in self.interruptUids) {
        if ([uid isEqualToString:interruptId]) {
            bFind = YES;
            break;
        }
    }
    
    if(bFind) {
        //如果主流断开
        if ([self.uid isEqualToString:interruptId]) {
            [self closeLiveCall];
        }
        else {
            [self removeLiveCallWithPlayUid:interruptId];
            [self sendCloseOneInviterRequestWithUid:interruptId];
            if ([self.currentInviterArray count] == 0) {
                [self.startLiveView.connectBtn setSelected:NO];
                [self.startLiveView removeAllChatViews];
                if (self.publiserVideoCall) {
                    int ret = [self.publiserVideoCall abortChat];
                    NSLog(@"结束连麦:%d", ret);
                }
                self.liveStatus = ALIVC_START_LIVE_STATUS_LIVING;
                return;
            }
        }
    }
}

//掉线情况
//0: 用户掉线
//1: server端掉线
- (void)onGetNetworkStatusMessage:(int)offlineType
{
    [self sendRequestLiveChatRoomInfoWithTopicName:self.mnsModel.topic Tags:@[self.mnsModel.roomTag, self.mnsModel.userRoomTag]];
}

#pragma mark - 直播SDK
/**
 *  创建直播
 */
- (void)createPublisher
{
    self.publiserVideoCall = [[AlivcVideoChatHost alloc] init];
    
    
    int width = 540;
    int height = 960;
    
    // 参数配置
    self.publisherParam  = [[NSMutableDictionary alloc] init];
    NSNumber* frontCamera = [[NSNumber alloc] initWithBool:cameraPositionFront];
    NSNumber* landscape = [[NSNumber alloc] initWithBool:NO];
    NSNumber* uploadTimeout = [[NSNumber alloc] initWithInt:5000];
    NSNumber* maxBitRate = [[NSNumber alloc] initWithInt:1500];
    NSNumber* cameraMirror = [[NSNumber alloc] initWithBool:YES];
    NSNumber* bitrate = [[NSNumber alloc] initWithInt:800];
    NSNumber* minBitrate = [[NSNumber alloc] initWithInt:400];

    [self.publisherParam setObject:maxBitRate forKey:ALIVC_PUBLISHER_PARAM_MAXBITRATE];
    [self.publisherParam setObject:minBitrate forKey:ALIVC_PUBLISHER_PARAM_MINBITRATE];
    [self.publisherParam setObject:bitrate forKey:ALIVC_PUBLISHER_PARAM_ORIGINALBITRATE];
    [self.publisherParam setObject:uploadTimeout forKey:ALIVC_PUBLISHER_PARAM_UPLOADTIMEOUT];
    [self.publisherParam setObject:landscape forKey:ALIVC_PUBLISHER_PARAM_LANDSCAPE];
    [self.publisherParam setObject:frontCamera forKey:ALIVC_PUBLISHER_PARAM_CAMERAPOSITION];
    [self.publisherParam setObject:cameraMirror forKey:ALIVC_PUBLISHER_PARAM_FRONTCAMERAMIRROR];
    
    // 预览
    int ret = [self.publiserVideoCall prepareToPublish:self.startLiveView.publisherView width:width height:height publisherParam:self.publisherParam];
    if (ret != 0) {
        NSLog(@"开启预览失败%d", ret);
    }
    // error Notification
    [self addVideoChatObserver];
    
    [UIApplication sharedApplication].idleTimerDisabled = YES;
    
    self.liveStatus = ALIVC_START_LIVE_STATUS_NONE;
}

/**
 *  关闭直播
 */
- (void)closeLive
{
    [self.publiserVideoCall stopPublishing];
    self.liveStatus = ALIVC_START_LIVE_STATUS_NONE;
}

/**
 *  离开直播（完全关闭）
 */
- (void)finishLive
{
    //正在连麦先结束连麦
    if (self.liveStatus == ALIVC_START_LIVE_STATUS_CHATTING) {
        [self closeLiveCall];
    }
    
    if (self.publiserVideoCall) {
        [self.publiserVideoCall stopPublishing];
        [self.publiserVideoCall finishPublishing];
        [self removeVideoChatObserver];
        self.publiserVideoCall = nil;
    }
    
    self.liveStatus = ALIVC_START_LIVE_STATUS_NONE;
}

/**
 *  连麦，打开多个连连麦方播放器
 */
- (void)createLiveCallWithInviteePlayUrls:(NSArray<NSURL*> *)inviteePlayUrlArray uids:(NSArray<NSString*> *)inviteePlayUidArray
{
    // 连麦
    [self.startLiveView.connectBtn setSelected:YES];
    
    self.playerParam = [[NSMutableDictionary alloc] init];
    
    //设置播放丢帧的时间阈值，缓冲区超过1s，则进行丢帧
    NSNumber* dropBufferDuration = [[NSNumber alloc] initWithInt:500];
    [self.playerParam setObject:dropBufferDuration forKey:ALIVC_PLAYER_PARAM_DROPBUFFERDURATION];
    
    //设置播放渲染模式
    NSNumber* scalingMode = [NSNumber numberWithInt:scalingModeAspectFitWithCropping];
    [self.playerParam setObject:scalingMode forKey:ALIVC_PLAYER_PARAM_SCALINGMODE];
    
    //设置播放下载超时时间
    NSNumber* downloadTimeout = [[NSNumber alloc] initWithInt:20000];
    [self.playerParam setObject:downloadTimeout forKey:ALIVC_PLAYER_PARAM_DOWNLOADIMEOUT];
    
    [self.publiserVideoCall setPlayerParam:self.playerParam];
    
    NSArray *inviteViews = [self.startLiveView addChatViewsWithArray:inviteePlayUrlArray uids:inviteePlayUidArray];
    
    int ret = [self.publiserVideoCall launchChats:inviteePlayUrlArray views:inviteViews];
    if (ret != 0) {
        [self showAlertViewWithMessage:[NSString stringWithFormat:@"SDK连麦初始化失败,ret=%d", ret]];
    }
    
    self.liveStatus = ALIVC_START_LIVE_STATUS_CHATTING;
}

/**
 *  添加连麦
 */
- (void)addLiveCallWithPlayUrls:(NSArray<NSURL *> *)playUrlArray uids:(NSArray<NSString*> *)playUidArray
{
    NSArray *inviteViews = [self.startLiveView addChatViewsWithArray:playUrlArray uids:playUidArray];
    int ret = [self.publiserVideoCall addChats:playUrlArray views:inviteViews];
    if (ret != 0) {
        [self showAlertViewWithMessage:@"SDK添加连麦失败"];
    }
    
    self.liveStatus = ALIVC_START_LIVE_STATUS_CHATTING;
}

- (void)removeLiveCallWithPlayUid:(NSString *)uid
{
    //找到移除的对应信息
    int removeIndex = -1;
    NSString* playUrl = nil;
    for (int index = 0; index < self.currentInviterArray.count; index++) {
        LiveInviteInfo *info = self.currentInviterArray[index];
        if ([info.uid isEqualToString:uid]) {
            removeIndex = index;
            playUrl = info.playUrl;
            break;
        }
    }
    
    if(removeIndex == -1){
        NSLog(@"未找到移除的连麦方: %@",uid);
        //assert(0);
        return;
    }
    
    [self.currentInviterArray removeObjectAtIndex:removeIndex];
    
    //改变窗口位置
    [self.startLiveView removeChatViewsWithUrl:playUrl];
    
    NSURL *playURL = [NSURL URLWithString:playUrl];
    int ret = [self.publiserVideoCall removeChats:@[playURL]];
    if (ret != 0) {
        [self showAlertViewWithMessage:@"SDK移除连麦失败"];
    }
    
    [self.inviterAudienceList removeObject:uid];
}

/**
 *  移除一个连麦
 */
- (void)removeLiveCallWithPlayurl:(NSString *)playUrl
{
    //找到移除的对应信息
    int removeIndex = -1;
    for (int index = 0; index < self.currentInviterArray.count; index++) {
        LiveInviteInfo *info = self.currentInviterArray[index];
        if ([info.playUrl isEqualToString:playUrl]) {
            removeIndex = index;
            break;
        }
    }
    
    if(removeIndex == -1){
        NSLog(@"未找到移除的连麦方: %@",playUrl);
        assert(0);
        return;
    }
    
    [self.currentInviterArray removeObjectAtIndex:removeIndex];
    
//    //移除最后一个连麦，则整个连麦关闭
//    if ([self.currentInviterArray count] == 0) {
//        [self closeLiveCall];
//        return;
//    }
    
    //改变窗口位置
    [self.startLiveView removeChatViewsWithUrl:playUrl];
    
    NSURL *playURL = [NSURL URLWithString:playUrl];
    int ret = [self.publiserVideoCall removeChats:@[playURL]];
    if (ret != 0) {
        [self showAlertViewWithMessage:@"SDK移除连麦失败"];
    }
}

/**
 *  结束连麦
 */
- (void)closeLiveCall
{
    [self.startLiveView.connectBtn setSelected:NO];
    [self.startLiveView removeAllChatViews];
    if (self.publiserVideoCall) {
        int ret = [self.publiserVideoCall abortChat];
        NSLog(@"结束连麦:%d", ret);
    }
    self.liveStatus = ALIVC_START_LIVE_STATUS_LIVING;
    [self.currentInviterArray removeAllObjects];
    [self.inviterAudienceList removeAllObjects];
    
    [self.interruptUids removeAllObjects];
}

/**
 *  调试信息
 */
- (void) updateInfo
{
    if(self.startLiveView.textView != nil ){
        AlivcPublisherPerformanceInfo* info = [self.publiserVideoCall getPublisherPerformanceInfo];
        
        NSMutableString *mutableString = [[NSMutableString alloc] init];
        int encode_speed = (info.videoEncodedBitrate+info.audioEncodedBitrate);
        int push_speed = (info.videoUploadedBitrate+info.audioUploadedBitrate);
        int video_queue_number = info.videoPacketsInBuffer;
        int audio_queue_number = info.audioPacketsInBuffer;
        [mutableString appendString:[NSString stringWithFormat:@"encode speed : %d kb/sec \n",encode_speed]];
        [mutableString appendString:[NSString stringWithFormat:@"push speed: %d kb/sec \n",push_speed]];
        [mutableString appendString:[NSString stringWithFormat:@"video frame number in queue : %d  \n",video_queue_number]];
        [mutableString appendString:[NSString stringWithFormat:@"audio frame number in queue : %d  \n",audio_queue_number]];
        [mutableString appendString:[NSString stringWithFormat:@"videoEncoderParamOfBitrate : %d \n",info.videoEncoderParamOfBitrate]];
        [mutableString appendString:[NSString stringWithFormat:@"currentlyUploadedVideoFramePts : %lld \n",info.currentlyUploadedVideoFramePts]];
        [mutableString appendString:[NSString stringWithFormat:@"currentlyUploadedAudioFramePts : %lld \n",info.currentlyUploadedAudioFramePts]];
        [mutableString appendString:[NSString stringWithFormat:@"previousKeyframePts : %lld \n",info.previousKeyframePts]];
        [mutableString appendString:[NSString stringWithFormat:@"totalFramesOfEncodedVideo : %lld \n",info.totalFramesOfEncodedVideo]];
        [mutableString appendString:[NSString stringWithFormat:@"totalTimeOfEncodedVideo : %lld \n",info.totalTimeOfEncodedVideo]];
        [mutableString appendString:[NSString stringWithFormat:@"totalTimeOfPublishing : %lld \n",info.totalTimeOfPublishing]];
        [mutableString appendString:[NSString stringWithFormat:@"totalFramesOfVideoUploaded : %lld \n",info.totalFramesOfVideoUploaded]];
        [mutableString appendString:[NSString stringWithFormat:@"totalSizeOfUploadedPackets : %lld \n",info.totalSizeOfUploadedPackets]];
        [mutableString appendString:[NSString stringWithFormat:@"dropDurationOfVideoFrames : %lld \n",info.dropDurationOfVideoFrames]];
        [mutableString appendString:[NSString stringWithFormat:@"videoDurationFromCaptureToUpload : %lld \n",info.videoDurationFromCaptureToUpload]];
        [mutableString appendString:[NSString stringWithFormat:@"audioDurationFromCaptureToUpload : %lld \n",info.audioDurationFromCaptureToUpload]];
        
        NSURL* playUrl = nil;
        if ([self.currentInviterArray count] > 0) {
            playUrl = [NSURL URLWithString:[self.currentInviterArray objectAtIndex:0].playUrl];
        }
        
        AlivcPlayerPerformanceInfo* playerInfo = [self.publiserVideoCall getPlayerPerformanceInfo:playUrl];
        if (playerInfo != nil) {
            [mutableString appendString:[NSString stringWithFormat:@"videoPacketsInBuffer : %d \n",playerInfo.videoPacketsInBuffer]];
            [mutableString appendString:[NSString stringWithFormat:@"audioPacketsInBuffer : %d \n",playerInfo.audioPacketsInBuffer]];
            [mutableString appendString:[NSString stringWithFormat:@"videoDurationFromDownloadToRender : %d \n",playerInfo.videoDurationFromDownloadToRender]];
            [mutableString appendString:[NSString stringWithFormat:@"audioDurationFromDownloadToRender : %d \n",playerInfo.audioDurationFromDownloadToRender]];
            [mutableString appendString:[NSString stringWithFormat:@"videoPtsOfLastPacketInBuffer : %lld \n",playerInfo.videoPtsOfLastPacketInBuffer]];
            [mutableString appendString:[NSString stringWithFormat:@"audioPtsOfLastPacketInBuffer : %lld \n",playerInfo.audioPtsOfLastPacketInBuffer]];
            [mutableString appendString:[NSString stringWithFormat:@"packetDownloadSpeed : %d \n",playerInfo.packetDownloadSpeed]];
        }
        
//        self.startLiveView.textView.text = mutableString;
    }
}


#pragma mark - Request
/**
 *  创建直播请求
 */
- (void)sendCreateLiveRequest
{
    [SendMessageManager creatLive:self.uid description:self.startLiveView.descTextFeild.text block:^(RoomInfoModel *roomInfo, MNSInfoModel *mnsInfo, NSError *error) {
        
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self showAlertViewWithMessage:[NSString stringWithFormat:@"创建直播失败error:%ld", (long)error.code]];
                [self.startLiveView.descButton setSelected:NO];
                self.startLiveView.descButton.userInteractionEnabled = YES;
                self.startLiveView.descButton.backgroundColor = kAlivcColor;
            });
            return ;
        }
        
        self.rtmpURLString = roomInfo.rtmpUrl;
        
//        self.rtmpURLString = @"rtmp://push-videocall.aliyuncs.com/test/516b6b8e4dd7?auth_key=1505476297-0-0-2934e3d7e6eb3f8e6ab3b1c13c0257ed";
        
        self.roomId = roomInfo.roomId;
        
        self.mnsModel = mnsInfo;
        if (!self.mnsModel) {
            AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"未请求到mnsModel,加入直播间失败" delegate:nil buttonTitles:@"OK",nil];
            [alert showInView:self.startLiveView];
            return ;
        }
        [self sendRequestLiveChatRoomInfoWithTopicName:self.mnsModel.topic Tags:@[self.mnsModel.roomTag, self.mnsModel.userRoomTag]];
    }];
}

/**
 *  获取直播间model请求
 */
- (void)sendRequestLiveChatRoomInfoWithTopicName:(NSString *)topic Tags:(NSArray *)tags
{
    [SendMessageManager getMnsTopicInfo:topic tags:tags block:^(AlivcWebSocketInfoModel *infoModel, NSError *error) {
        
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                
                if (error.code < 0) {
                    self.isNeedReconnectIM = YES;
                    return ;
                }
                
                [self showAlertViewWithMessage:[NSString stringWithFormat:@"获取info Model error:%ld", (long)error.code]];
            });
            return ;
        }
        
        [SendMessageManager joinChatRoomWithWebSocketInfoModel:infoModel success:^{
            NSLog(@"加入直播间成功");
            self.isNeedReconnectIM = NO;
            
            [self.startLiveView.descView removeFromSuperview];
            
            int ret = [self.publiserVideoCall startToPublish:self.rtmpURLString];
            if (ret == 0) {
                NSTimer *timer = [NSTimer timerWithTimeInterval:1 target:self selector:@selector(updateInfo) userInfo:nil repeats:YES];
                [[NSRunLoop mainRunLoop] addTimer:timer forMode:NSDefaultRunLoopMode];
            }
            self.liveStatus = ALIVC_START_LIVE_STATUS_LIVING;
            
        } error:^(NSError *error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"聊天SDK加入直播间失败error:%ld", (long)error.code]];
        }];
    }];
}

/**
 *  关闭直播请求
 */
- (void)sendCloseLiveRequest
{
    [SendMessageManager leaveLive:self.roomId block:^(NSError *error) {
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self showAlertViewWithMessage:[NSString stringWithFormat:@"关闭直播失败error:%ld", (long)error.code]];
            });
            return ;
        }
    }];
}

/**
 *  发起连麦
 */
- (void)sendInviteVideoCallWithInviterUids:(NSArray<NSString *> *)inviteeUidArray
{
    [SendMessageManager inviteVideoCall:self.roomId inviterUid:self.uid inviteeUids:inviteeUidArray inviterType:2 block:^(NSError *error) {
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self showAlertViewWithMessage:[NSString stringWithFormat:@"发起连麦请求失败error:%ld", (long)error.code]];
            });
            return ;
        }
    }];
}

/**
 *  发送获取连麦消息
 */
- (void)sendInviteVideoCallFeedbackRequestWithUid:(NSString *)inviterUid status:(int)status inviterType:(int)inviterType
{
    [SendMessageManager sendVideoCallFeedBack:inviterUid inviteeUid:self.uid status:status inviterType:inviterType inviteeType:2 block:^(NSURL *mainPlayUrl, NSArray<NSURL *> *playUrls, NSArray<NSString *> *playUids, NSString *rtmpUrl, NSError *error) {
        
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"连麦反馈请求失败%ld", (long)error.code]];
            return ;
        }
    }];
}

/**
 *  移除某一个连麦请求
 */
- (void)sendCloseOneInviterRequestWithUid:(NSString *)uid
{
    [SendMessageManager leaveVideoCall:self.roomId uid:uid block:^(NSError *error) {
        if (error) {
            [self showAlertViewWithMessage:[NSString stringWithFormat:@"离开连麦请求失败error:%ld", (long)error.code]];
            return ;
        }
    }];
}

/**
 *  关闭连麦请求
 */
- (void)sendCloseInviteRequest {
   
    [SendMessageManager closeVideoCall:self.roomId block:^(NSError *error) {
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self showAlertViewWithMessage:[NSString stringWithFormat:@"关闭连麦请求失败error:%ld", (long)error.code]];
            });
            return ;
        }
    }];
}

/**
 *  获取观众列表
 */
- (void)requestCustomsList {
    
    [self.view bringSubviewToFront:self.startLiveView.chatListView];
    self.startLiveView.chatListView.hidden = NO;

    // 小菊花
    MBProgressHUD *mbHub = [MBProgressHUD showHUDAddedTo:self.startLiveView.chatListView animated:YES];
    mbHub.label.text = @"loading...";
    
    [SendMessageManager getRoomUsers:self.roomId block:^(NSMutableArray *userInfos, NSError *error) {
        
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [mbHub hideAnimated:YES];
                self.startLiveView.chatListView.hidden = YES;
                [self showAlertViewWithMessage:[NSString stringWithFormat:@"获取当前观众列表失败error:%ld", (long)error.code]];
            });
            return ;
        }
        
        [self.audienceListArray removeAllObjects];
        for (UserInfo* userInfo in userInfos) {
            
            BOOL bFind = NO;
            for (LiveInviteInfo* info in self.currentInviterArray) {
                if ([info.uid isEqualToString:userInfo.id]) {
                    bFind = YES;
                    break;
                }
            }
            if (bFind) {
                continue;
            }
            
            [self.audienceListArray addObject:userInfo];
        }
        
        [self.startLiveView.chatListView.tableView_audience reloadData];
        [self.startLiveView.chatListView.tableView_videocall reloadData];
        [mbHub hideAnimated:YES];
        
    }];
}

// CDN推流连麦异常消息处理
- (void)mixMessageClassifyWithMessage:(NSString *)code mainRoomId:(NSString*)mainRoomId
{
    static BOOL isAnomalyTimeout = NO; // 是否重连超时
    
    //  当前 是否是主流
    BOOL isMain = [self.roomId isEqualToString:mainRoomId];
    
    NSString *remindMessage = @"";
    
    if ([code isEqualToString:@"Success"]) {
        
        // 混流成功
        remindMessage = @"连麦进行中";
        isAnomalyTimeout = NO;
        return;
        
    } else if ([code isEqualToString:@"MainStreamNotExist"]) {
        
        // 主流不存在 混流失败
        if (isMain) {
            // 提示 当前用户断流
            remindMessage = @"您的推流暂时中断,正在恢复中...";
        } else {
            // 提示 对方断流
            remindMessage = @"对方推流暂时中断,正在恢复中...";
        }
        isAnomalyTimeout = YES;
        
    } else if ([code isEqualToString:@"MixStreamNotExist"]) {
        
        // 副流不存在 混流失败
        if (isMain) {
            // 提示 对方断流
            remindMessage = @"对方推流暂时中断,正在恢复中...";
        } else {
            // 提示 当前断流
            remindMessage = @"您的推流暂时中断,正在恢复中...";
        }
        isAnomalyTimeout = YES;
        
    } else if ([code isEqualToString:@"InternalError"]) {
        
        // 流内部错误 混流失败
        remindMessage = @"CDN处理内部错误InternalError,正在恢复中...";
        isAnomalyTimeout = YES;
        
    } else {
        // 无  未知错误
        remindMessage = @"连麦中断,未知错误,无对应错误码";
        
        isAnomalyTimeout = YES;
    }
    
    if (remindMessage.length > 0 && (self.liveStatus == ALIVC_START_LIVE_STATUS_CHATTING || self.liveStatus == ALIVC_START_LIVE_STATUS_CALLING)) {
        [self showAlertViewWithMessage:remindMessage];
    }
    
    if (isAnomalyTimeout) {
        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(20.0 * NSEC_PER_SEC)), dispatch_get_main_queue(),^{
            if (isAnomalyTimeout && (self.liveStatus == ALIVC_START_LIVE_STATUS_CHATTING || self.liveStatus == ALIVC_START_LIVE_STATUS_CALLING)) {
                AlivcLiveAlertView *alertView = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"等待重连超时，结束连麦" delegate:nil buttonTitles:@"OK", nil];
                [alertView showInView:self.startLiveView];
                if (self.startLiveView.connectBtn.isSelected) {
                    [self connectOrStopButtonAction:self.startLiveView.connectBtn];
                }
            }
        });
    }
}

#pragma mark - StartViewDelegate
- (void)onClickChatViewCloseButtonWithUrl:(NSString *)playUrl
{
    int findIndex = -1;
    for (int i=0; i<(int)[self.currentInviterArray count]; i++) {
        LiveInviteInfo* info = [self.currentInviterArray objectAtIndex:i];
        if (info) {
            if ([playUrl isEqualToString:info.playUrl]) {
                findIndex = i;
                break;
            }
        }
    }
    if (findIndex == -1) {
        return;
    }
    
    LiveInviteInfo *info = self.currentInviterArray[findIndex];
    [self sendCloseOneInviterRequestWithUid:info.uid];
}

#pragma mark - UITableViewDelegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if ([tableView isEqual:self.startLiveView.chatTableView]) {
        return self.chatArray.count;
    }
    else if([tableView isEqual:self.startLiveView.chatListView.tableView_videocall]){
        return self.currentInviterArray.count;
    }
    else if([tableView isEqual:self.startLiveView.chatListView.tableView_audience]){
        return self.audienceListArray.count;
    }
    
    return 0;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 64;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if ([tableView isEqual:self.startLiveView.chatTableView]) {
        UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"chatTableViewID" forIndexPath:indexPath];
        
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.backgroundColor = [UIColor clearColor];
        
        UIView *bgView = [[UIView alloc] initWithFrame:cell.contentView.bounds];
        UILabel *label = [[UILabel alloc] initWithFrame:(CGRectMake(0, 2, CGRectGetWidth(cell.contentView.frame), CGRectGetHeight(cell.contentView.frame) - 4))];
        label.backgroundColor = [UIColor colorWithWhite:0 alpha:0.2];
        label.textColor = kAlivcColor;
        label.font = [UIFont fontWithName:@"Arial" size:14.0];
        [label.layer setMasksToBounds:YES];
        [label.layer setCornerRadius:12.0];
        label.text = self.chatArray[indexPath.row];
        [bgView addSubview:label];
        [cell setBackgroundView:bgView];
        
        return cell;
    } else if ([tableView isEqual:self.startLiveView.chatListView.tableView_audience]){
        ALiViewTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"liveListTableViewID" forIndexPath:indexPath];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.cellModelDelegate = self;
        NSString *uid = self.audienceListArray[indexPath.row].id;
        ALiCustomListCellModel* mode = [[ALiCustomListCellModel alloc] init];
        mode.userName = [NSString stringWithFormat:@"%@(ID:%@)", self.audienceListArray[indexPath.row].name, uid];
        mode.isChanged = NO;
        [cell setModel:mode];
        return cell;
    }
    else if ([tableView isEqual:self.startLiveView.chatListView.tableView_videocall]){
        ALiViewTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"liveListTableViewID" forIndexPath:indexPath];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.cellModelDelegate = self;
        NSString *uid = self.currentInviterArray[indexPath.row].uid;
        ALiCustomListCellModel* mode = [[ALiCustomListCellModel alloc] init];
        mode.userName = [NSString stringWithFormat:@"%@(ID:%@)", self.currentInviterArray[indexPath.row].name, uid];
        mode.isChanged = NO;
        [cell setModel:mode];
        return cell;
    }
    
    return nil;
}

-(void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    cell.backgroundColor = [UIColor clearColor];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    cell.backgroundColor = [UIColor colorWithRed:(33)/255.0 green:(33)/255.0 blue:(33)/255.0 alpha:0.5];
}

#pragma mark - AlivcLiveAlertViewDelegate

- (void)alertView:(AlivcLiveAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (alertView.tag == ALIVC_START_ALERT_TAG_VIDEOCALL_INVITE) {
        NSString* inviterUid = alertView.message;
        
        if (buttonIndex == 0) {
            // 不同意连麦
            [self sendInviteVideoCallFeedbackRequestWithUid:inviterUid status:2 inviterType:1];
        }
        if (buttonIndex == 1) {
            // 同意连麦  如果没有推流先开始推流
            if (!self.startLiveView.pushButton.selected) {
                [self startPush:self.startLiveView.pushButton];
            }
            
            [self sendInviteVideoCallFeedbackRequestWithUid:inviterUid status:1 inviterType:1];
        }
    }
    
    //退出连麦
    if (alertView.tag == ALIVC_START_ALERT_TAG_VIDEOCALL_EXIT) {
        if (buttonIndex == 1) {
            if (self.startLiveView.connectBtn.selected) {
                [self connectOrStopButtonAction:self.startLiveView.connectBtn];
            }
        }
    }
    
    if (alertView.tag == ALIVC_START_ALERT_TAG_ROOM_EXIT) {
        [self DonePressed:self.startLiveView.doneBtn];
    }
}


#pragma mark - NSNotification
- (void) addVideoChatObserver
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPublishConnectError:)
                                                 name:AlivcVideoChatPublisherOpenFailed object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnNetworkTimeout:)
                                                 name:AlivcVideoChatPublisherSendDataTimeout object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoCaptureError:)
                                                 name:AlivcVideoChatPublisherVideoCaptureDisabled object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnAudioCaptureError:)
                                                 name:AlivcVideoChatPublisherAudioCaptureDisabled object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoInitError:)
                                                 name:AlivcVideoChatPublisherVideoEncoderInitFailed object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnAudioInitError:)
                                                 name:AlivcVideoChatPublisherAudioEncoderInitFailed object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnMemoryMallocError:)
                                                 name:AlivcVideoChatMemoryPool object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnNetworkSlow:)
                                                 name:AlivcVideoChatPublisherNetSpeedPoor object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnAudioEncodeError:)
                                                 name:AlivcVideoChatPublisherEncodeAudioFailed object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoEncodeError:)
                                                 name:AlivcVideoChatPublisherEncodeVideoFailed object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoChatStartBuffering:)
                                                 name:AlivcVideoChatPlayerStartBuffering object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnVideoChatEndBuffering:)
                                                 name:AlivcVideoChatPlayerEndBuffering object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onChatHostDisplayFirstFrame:)
                                                 name:AlivcVideoChatPlayerFirstFrameRender object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPlayerReadPacketTimeout:)
                                                 name:AlivcVideoChatPlayerReadPacketTimeout object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPlayerNoDisplayViewer:)
                                                 name:AlivcVideoChatPlayerNoDisplayViewer object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPlayerInvalidCodec:)
                                                 name:AlivcVideoChatPlayerInvalidCodec object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(OnPlayerOpenFailed:)
                                                 name:AlivcVideoChatPlayerOpenFailed object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onAudioCaptureError:) name:AlivcVideoChatPublisherAudioCaptureError object:self.publiserVideoCall];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onCaptureFpsSlow:) name:AlivcVideoChatPublisherCaptureFpsSlow object:self.publiserVideoCall];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillResignActive:)
                                                 name:UIApplicationWillResignActiveNotification object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationDidBecomeActive:)
                                                 name:UIApplicationDidBecomeActiveNotification object:nil];

}

-(void)removeVideoChatObserver
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)OnPublishConnectError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"推流连接断开" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    [self.startLiveView.pushButton setSelected:NO];
}

- (void)OnNetworkTimeout:(NSNotification *)notification
{    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"推流连接超时,请重新连接（如果在连麦中将自动结束连麦）" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];

    [alert show];
    
    if (self.startLiveView.connectBtn.selected) {
        [self connectOrStopButtonAction:self.startLiveView.connectBtn];
    }
    
    if (self.startLiveView.pushButton.selected) {
        [self startPush:self.startLiveView.pushButton];
    }
}

- (void)OnVideoCaptureError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"视频采集被禁止" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
}

- (void)OnAudioCaptureError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"音频采集被禁止" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
}

- (void)OnVideoInitError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"视频初始化失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.startLiveView.connectBtn.selected) {
        [self connectOrStopButtonAction:self.startLiveView.connectBtn];
    }
}


- (void)OnAudioInitError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"音频初始化失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.startLiveView.connectBtn.selected) {
        [self connectOrStopButtonAction:self.startLiveView.connectBtn];
    }
}

- (void)OnMemoryMallocError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"内存分配失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.startLiveView.connectBtn.selected) {
        [self connectOrStopButtonAction:self.startLiveView.connectBtn];
    }
}

- (void)OnNetworkSlow:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"网速太慢,建议中断直播" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
}

- (void)OnAudioEncodeError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"音频编码失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.startLiveView.connectBtn.selected) {
        [self connectOrStopButtonAction:self.startLiveView.connectBtn];
    }
}

- (void)OnVideoEncodeError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"视频编码失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    [alert show];
    
    if (self.startLiveView.connectBtn.selected) {
        [self connectOrStopButtonAction:self.startLiveView.connectBtn];
    }
}

- (void)OnVideoChatStartBuffering:(NSNotification *)notification
{
    NSLog(@"video chat start to buffering\n");
}

- (void)OnVideoChatEndBuffering:(NSNotification *)notification
{
    NSLog(@"video chat end buffering\n");
}

- (void)onChatHostDisplayFirstFrame:(NSNotification *)notification
{
    NSLog(@"播放首帧");
}

- (void)OnPlayerReadPacketTimeout:(NSNotification *)notification
{
    NSString* playUrl = nil;
    NSDictionary* diction = [notification userInfo];
    if (diction) {
        playUrl = [diction objectForKey:@"playUrl"];
    }
    
    if (playUrl == nil) {
        assert(0);
    }
    
    AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:playUrl icon:nil message:@"重连中，请稍后" delegate:self buttonTitles:@"OK", nil];
    [alert showInView:self.startLiveView];
    
    if (self.startLiveView.connectBtn.selected) {
        [self.publiserVideoCall reconnectChat:[NSURL URLWithString:playUrl]];
    }
}

- (void)OnPlayerNoDisplayViewer:(NSNotification *)notification
{
    NSString* playUrl = nil;
    NSDictionary* diction = [notification userInfo];
    if (diction) {
        playUrl = [diction objectForKey:@"playUrl"];
    }
    
    if (playUrl == nil) {
        assert(0);
    }
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:playUrl message:@"播放无显示窗口" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}

- (void)OnPlayerInvalidCodec:(NSNotification *)notification
{
    NSString* playUrl = nil;
    NSDictionary* diction = [notification userInfo];
    if (diction) {
        playUrl = [diction objectForKey:@"playUrl"];
    }
    
    if (playUrl == nil) {
        assert(0);
    }
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:playUrl message:@"无效的解码器" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}

- (void)OnPlayerOpenFailed:(NSNotification *)notification
{
    NSString* playUrl = nil;
    NSDictionary* diction = [notification userInfo];
    if (diction) {
        playUrl = [diction objectForKey:@"playUrl"];
    }
    
    if (playUrl == nil) {
        assert(0);
    }
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:playUrl message:@"播放打开失败" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}

- (void)onAudioCaptureError:(NSNotification *)notification
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"错误" message:@"音频采集错误，直播退出" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    alert.tag = ALIVC_START_ALERT_TAG_ROOM_EXIT;
    [alert show];
    
    if (self.startLiveView.connectBtn.selected) {
        [self connectOrStopButtonAction:self.startLiveView.connectBtn];
    }
}

- (void)onCaptureFpsSlow:(NSNotification *)notification
{
    NSLog(@"视频采集帧率低");
}

-(void)applicationWillResignActive:(NSNotification *)notification
{
    
    [self.publiserVideoCall pause];
}

-(void)applicationDidBecomeActive:(NSNotification *)notification
{
    [self.publiserVideoCall resume];
}

- (NSString *)currentTime
{
    NSDate * date = [NSDate date];
    NSTimeInterval sec = [date timeIntervalSinceNow];
    NSDate * currentDate = [[NSDate alloc] initWithTimeIntervalSinceNow:sec];
    
    NSDateFormatter * df = [[NSDateFormatter alloc] init];
    [df setDateFormat:@"HH:mm:ss"];
    
    return [df stringFromDate:currentDate];
}

#pragma mark - Actions

- (void)addButtonActions
{
    [self.startLiveView.pushButton addTarget:self action:@selector(startPush:) forControlEvents:UIControlEventTouchUpInside];
    [self.startLiveView.skinButton addTarget:self action:@selector(skinButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.startLiveView.closeAudio addTarget:self action:@selector(closeAudioButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.startLiveView.toggleButton addTarget:self action:@selector(toggleButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.startLiveView.descButton addTarget:self action:@selector(descButtonAction:) forControlEvents:(UIControlEventTouchUpInside)];
    [self.startLiveView.doneBtn addTarget:self action:@selector(DonePressed:) forControlEvents:UIControlEventTouchUpInside];
    [self.startLiveView.connectBtn addTarget:self action:@selector(connectOrStopButtonAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.startLiveView.addButton addTarget:self action:@selector(addButtonClick:) forControlEvents:(UIControlEventTouchUpInside)];
    [self.startLiveView.chatListView.backBtn addTarget:self action:@selector(listViewCancel:) forControlEvents:(UIControlEventTouchUpInside)];
    //[self.startLiveView.liveListOkButton addTarget:self action:@selector(listOkClick:) forControlEvents:(UIControlEventTouchUpInside)];
    [self.startLiveView.liveListCancelButton addTarget:self action:@selector(listCancelClick:) forControlEvents:(UIControlEventTouchUpInside)];
    [self.startLiveView.chatListView.button addTarget:self action:@selector(listOkClick:) forControlEvents:(UIControlEventTouchUpInside)];
}

- (void)startPush:(UIButton *)sender
{
    if (sender.selected) {
        // 结束推流
        if (self.startLiveView.connectBtn.selected) {
            // 如果连麦中的话 提示先结束连麦
            [self showAlertViewWithMessage:@"连麦中，请先结束连麦"];
            return;
        } else {
            [self closeLive];
            self.liveStatus = ALIVC_START_LIVE_STATUS_NONE;
        }
        
    } else {
        // 开始推流
        if(!self.rtmpURLString) {
            NSLog(@"未获取推流URL");
            return;
        }
        
        int ret = [self.publiserVideoCall startToPublish:self.rtmpURLString];
        if (ret == 0) {
            NSTimer *timer = [NSTimer timerWithTimeInterval:1 target:self selector:@selector(updateInfo) userInfo:nil repeats:YES];
            [[NSRunLoop mainRunLoop] addTimer:timer forMode:NSDefaultRunLoopMode];
        }
        self.liveStatus = ALIVC_START_LIVE_STATUS_LIVING;
    }
    
    [sender setSelected:!sender.selected];
}

- (void)DonePressed:(UIButton *)button
{
    if (!self.roomId) {
        [self finishLive];
        [self dismissViewControllerAnimated:YES completion:nil];
        [UIApplication sharedApplication].idleTimerDisabled = NO;
        return;
    }
    
    if (self.startLiveView.connectBtn.selected) {
        AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:@"当前正在连麦中，请先结束连麦，然后在退出直播" delegate:self buttonTitles:@"取消", @"结束连麦", nil];
        alert.tag = ALIVC_START_ALERT_TAG_VIDEOCALL_EXIT;
        [alert showInView:self.startLiveView];
        return;
    }
    
    //发送退出直播请求
    [self sendCloseLiveRequest];
    
    [SendMessageManager quitChatRoomSuccess:^{
        NSLog(@"SDK退出直播间成功");
    } error:^(NSError *error) {
        
    }];
    
    [self finishLive];
    [UIApplication sharedApplication].idleTimerDisabled = NO;
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)connectOrStopButtonAction:(UIButton *)sender
{
    if (sender.selected) {
        // 结束连麦
        [self closeLiveCall];
        [self sendCloseInviteRequest];
    } else {
        // 发起连麦
        if (!self.startLiveView.pushButton.selected) {
            [self showAlertViewWithMessage:@"请先开始推流"];
            return;
        }
        
        //请求观众列表
        [self requestCustomsList];
    }
}

- (void)listViewCancel:(UIButton *)sender
{
    self.startLiveView.chatListView.hidden = YES;
}

- (void)addButtonClick:(UIButton *)sender
{
    [self requestCustomsList];
}

- (void)listCancelClick:(UIButton *)sender
{
    //self.startLiveView.liveListView.hidden = YES;
}

- (void)listOkClick:(UIButton *)sender
{
    NSMutableArray* array = [[NSMutableArray alloc] init];
    
    if ([self.startLiveView.chatListView isToVideoCall]) {
        int count = (int)self.audienceListArray.count;
        for (int i=0; i<count; i++) {
            
            ALiViewTableViewCell *cell = [self.startLiveView.chatListView.tableView_audience cellForRowAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0]];
            if (cell) {
                if ([cell isChecked]) {
                    [array addObject:self.audienceListArray[i].id];
                    LiveInviteInfo* info = [[LiveInviteInfo alloc] init];
                    info.uid =self.audienceListArray[i].id;
                    [self.inviterAudienceList addObject:info.uid];
                }
            }
        }
        
        if ([array count] == 0) {
            return;
        }
        
        if ([self.currentInviterArray count] + [array count] > 3) {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"提示" message:@"已超过最大连麦人数" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
            
            [alert show];
            return;
        }
        
        [self sendInviteVideoCallWithInviterUids:array];
        
        self.liveStatus = ALIVC_START_LIVE_STATUS_CALLING;
    }
    else{
        int count = (int)self.currentInviterArray.count;
        for (int i=0; i<count; i++) {
            
            ALiViewTableViewCell *cell = [self.startLiveView.chatListView.tableView_videocall cellForRowAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0]];
            if (cell) {
                if ([cell isChecked]) {
                    [array addObject:self.currentInviterArray[i].uid];
                }
            }
        }
        
        if ([array count] == 0) {
            return;
        }
        
        for (NSString* uid in array) {
            [self removeLiveCallWithPlayUid:uid];
            [self sendCloseOneInviterRequestWithUid:uid];
        }
        
        //移除最后一个连麦，则整个连麦关闭
        if ([self.currentInviterArray count] == 0) {
            [self.startLiveView.connectBtn setSelected:NO];
            [self.startLiveView removeAllChatViews];
            if (self.publiserVideoCall) {
                int ret = [self.publiserVideoCall abortChat];
                NSLog(@"结束连麦:%d", ret);
            }
            self.liveStatus = ALIVC_START_LIVE_STATUS_LIVING;
        }
    }
    
    
    self.startLiveView.chatListView.hidden = YES;
}

- (void)skinButtonAction:(UIButton *)sender
{
    NSNumber* number = [[NSNumber alloc] initWithBool:sender.selected];
    NSString* key = ALIVC_FILTER_PARAM_BEAUTY_ON;
    NSDictionary* dic = [[NSDictionary alloc] initWithObjectsAndKeys:number, key,nil];
    [self.publiserVideoCall setFilterParam:dic];
    [sender setSelected:!sender.selected];
}

- (void)closeAudioButtonAction:(UIButton *)sender {
    if (sender.selected) {
        self.publiserVideoCall.publisherMuteMode = NO;
    } else {
        self.publiserVideoCall.publisherMuteMode = YES;
    }
    [sender setSelected:!sender.selected];
}


- (void)toggleButtonAction:(UIButton *)sender
{
    [self.publiserVideoCall switchCamera];
}

- (void)descButtonAction:(UIButton *)sender
{
//    if (self.startLiveView.descTextFeild.text.length == 0) {
//        self.startLiveView.descTextFeild.placeholder = @"直播名称不能为空";
//        return;
//    }
//    [sender setSelected:YES];
//    sender.userInteractionEnabled = NO;
//    sender.backgroundColor = [UIColor grayColor];
//    [self sendCreateLiveRequest];
//    [self.startLiveView.descTextFeild resignFirstResponder];
//    self.startLiveView.descTextFeild.placeholder = @"请输入直播名称";
    
    [self.startLiveView createLiveChangeView];
    
    [self sendCreateLiveRequest];
    
    [self.startLiveView.descTextFeild resignFirstResponder];
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    [self.startLiveView.descTextFeild resignFirstResponder];
}

#pragma mark - AlertView
- (void)showAlertViewWithMessage:(NSString *)message
{
    AlivcLiveAlertView *alert = [[AlivcLiveAlertView alloc] initWithTitle:@"提示" icon:nil message:message delegate:self buttonTitles:@"OK", nil];
    [alert showInView:self.startLiveView];
}


@end
