//
//  AppServerCommunication.m
//  DemoAlivcLive
//
//  Created by shiping.csp on 2017/1/11.
//  Copyright © 2017年 Alivc. All rights reserved.
//

#import "SendMessageManager.h"
#import "AlivcRequestManager.h"
#import "RoomInfoModel.h"

@implementation SendMessageManager

+(void) userLogIn:(NSString*)userName password:(NSString*)password block:(void (^)(UserInfo* userInfo, NSError *error))block
{
    NSDictionary *param = @{@"name":userName};
    [[AlivcRequestManager manager] requestWithHost:@"login" param:param block:^(NSDictionary *response, NSError *error) {
        
        if (error) {
            if(block)
                block(nil,error);
            return ;
        };
        
        UserInfo* userInfo = [[UserInfo alloc] initWithDictionary:response error:nil];
        
        if(block)
            block(userInfo,error);
    }];
}

+(void) getRoomUsers:(NSString*)roomId block:(void (^)(NSMutableArray<UserInfo*>* userInfos, NSError *error))block
{
    NSDictionary *param = @{@"roomId":roomId};
    [[AlivcRequestManager manager] requestWithHost:@"im/room/users" param:param block:^(NSDictionary *response, NSError *error) {
        
        if (error) {
            if(block)
                block(nil,error);
            return ;
        };
        
        NSMutableArray* array = [[NSMutableArray alloc] init];
        for (NSDictionary *dictionary in response) {
            UserInfo *model = [[UserInfo alloc] initWithDictionary:dictionary error:nil];
            [array addObject:model];
        }
        
        if(block)
            block(array,error);
        
    }];
}

+(void) getAppLiveList:(void (^)(NSMutableArray<RoomInfoModel*>* roomInfos,NSError *error))block
{
    [[AlivcRequestManager manager] requestWithHost:@"live/list" param:nil block:^(NSDictionary *response, NSError *error) {
        
        if (error) {
            if(block)
                block(nil,error);
            return ;
        };
        
        NSMutableArray* array = [[NSMutableArray alloc] init];
        for (NSDictionary *dictionary in response) {
            RoomInfoModel *model = [[RoomInfoModel alloc] initWithDictionary:dictionary error:nil];
            [array addObject:model];
        }
        
        if(block)
            block(array,error);
    }];
}

+(void) creatLive:(NSString*)uid description:(NSString*)description block:(void (^)(RoomInfoModel* roomInfo, MNSInfoModel* mnsInfo, NSError *error))block
{
    NSDictionary *param = @{@"uid":uid,
                            @"desc":description};
    [[AlivcRequestManager manager] requestWithHost:@"live/create" param:param block:^(NSDictionary *response, NSError *error) {
    
        if (error) {
            if(block)
                block(nil,nil,error);
            return ;
        };
        
        RoomInfoModel* roomModel = [[RoomInfoModel alloc] initWithDictionary:response error:nil];
        MNSInfoModel* mnsModel = [[MNSInfoModel alloc] initWithDictionary:[response objectForKey:@"mns"] error:nil];
        
        if(block)
            block(roomModel,mnsModel,error);

    }];
}

+(void)watchLive:(NSString*)uid roomId:(NSString*)roomId block:(void (^)(MNSInfoModel* mnsInfo, NSError *error))block
{
    NSDictionary *param = @{@"roomId":roomId,
                            @"uid":uid};
    [[AlivcRequestManager manager] requestWithHost:@"live/play" param:param block:^(NSDictionary *response, NSError *error) {
       
        if (error) {
            if(block)
                block(nil,error);
            return ;
        };
        
        MNSInfoModel* mnsModel = [[MNSInfoModel alloc] initWithDictionary:[response objectForKey:@"mns"] error:nil];
        
        if(block)
            block(mnsModel,error);

    }];
}

+(void)likeWatch:(NSString*)uid roomId:(NSString*)roomId block:(void (^)(NSError *error))block
{
    NSDictionary *param = @{@"uid":uid,
                            @"roomId":roomId};
    
    [[AlivcRequestManager manager] requestWithHost:@"live/like" param:param block:^(NSDictionary *response, NSError *error) {
        
        if (block) {
            block(error);
        }
    }];
}

+(void)leaveLive:(NSString*)roomId block:(void (^)(NSError *error))block
{
    NSDictionary *param = @{@"roomId":roomId};
    [[AlivcRequestManager manager] requestWithHost:@"live/leave" param:param block:^(NSDictionary *response, NSError *error) {
        
        if(block)
            block(error);
        
    }];
}

+(void)sendLiveComments:(NSString*)roomId uid:(NSString*)uid comments:(NSString*)comments block:(void (^)(NSError *error))block
{
    NSDictionary *param = @{@"uid":uid,
                            @"roomId":roomId,
                            @"comment":comments};
    [[AlivcRequestManager manager] requestWithHost:@"live/comment" param:param block:^(NSDictionary *response, NSError *error) {
        
        if(block) {
            block(error);
        }
        
    }];
}

+(void)leaveWatch:(NSString*)roomId uid:(NSString*)uid block:(void (^)(NSError *error))block
{
    NSDictionary *param = @{@"roomId":roomId,
                            @"uid":uid};
    [[AlivcRequestManager manager] requestWithHost:@"live/audience/leave" param:param block:^(NSDictionary *response, NSError *error) {
        
        if(block) {
            block(error);
        }
        
    }];
}

+(void)inviteVideoCall:(NSString*)roomId inviterUid:(NSString*)inviterUid inviteeUids:(NSArray<NSString*>*)inviteeUids inviterType:(int)inviterType block:(void (^)(NSError *error))block
{
    NSMutableString* inviteeUid = [[NSMutableString alloc] init];
    int count = (int)[inviteeUids count];
    int index = 0;
    for (NSString* uid in inviteeUids) {
        [inviteeUid appendString:uid];
        index++;
        if (index < count) {
            [inviteeUid appendString:@"|"];
        }
    }
    
    NSDictionary *param = @{@"inviterUid":inviterUid,
                            @"inviteeUids":inviteeUid,
                            @"liveRoomId":roomId,
                            @"inviterType":@(inviterType)};
    
    [[AlivcRequestManager manager] requestWithHost:@"videocall/invite" param:param block:^(NSDictionary *response, NSError *error) {
        
        if(block)
            block(error);
        
    }];
}

+(void)sendVideoCallFeedBack:(NSString*)inviterUid inviteeUid:(NSString*)inviteeUid status:(int)status inviterType:(int)inviterType inviteeType:(int)inviteeType block:(void (^)(NSURL* mainPlayUrl,NSArray<NSURL*>* playUrls,NSArray<NSString*>* Uids,NSString* rtmpUrl,NSError *error))block
{
    NSDictionary *param = @{@"inviterUid":inviterUid,
                            @"inviteeUid":inviteeUid,
                            @"status":@(status),
                            @"inviterType":@(inviterType),
                            @"inviteeType":@(inviteeType)};
    
    [[AlivcRequestManager manager] requestWithHost:@"videocall/feedback" param:param block:^(NSDictionary *response, NSError *error) {
        if (error) {
            if(block)
                block(nil,nil,nil,nil,error);
            return ;
        }
        
        NSURL *mainPlayUrl = [NSURL URLWithString:response[@"mainPlayUrl"]];
        NSArray *otherPlayUrlArray = response[@"playUrls"];
        NSMutableArray* otherPlayUrls = [[NSMutableArray alloc] init];
        NSMutableArray* otherPlayUids = [[NSMutableArray alloc] init];
        for (NSDictionary* dic in otherPlayUrlArray) {
            NSString* uid = dic[@"uid"];
            NSString* strUid = [NSString stringWithFormat:@"%d",[uid intValue]];
            [otherPlayUids addObject:strUid];
            [otherPlayUrls addObject:[NSURL URLWithString:dic[@"url"]]];
        }
        NSString *pushUrl = response[@"rtmpUrl"];
        if (block) {
            block(mainPlayUrl,otherPlayUrls,otherPlayUids,pushUrl,error);
        }
    }];
}

+(void)closeVideoCall:(NSString*)roomId block:(void (^)(NSError *error))block
{
    NSDictionary *param = @{@"roomId":roomId};
    [[AlivcRequestManager manager] requestWithHost:@"videocall/close" param:param block:^(NSDictionary *response, NSError *error) {
        if (block) {
            block(error);
        }
    }];
}

+(void)leaveVideoCall:(NSString*)roomId uid:(NSString*)uid block:(void (^)(NSError *error))block
{
    NSDictionary *param = @{@"roomId":roomId,
                            @"uid":uid};
    [[AlivcRequestManager manager] requestWithHost:@"videocall/leave" param:param block:^(NSDictionary *response, NSError *error) {
        if (block) {
            block(error);
        }
    }];
}

+(void)getMnsTopicInfo:(NSString*)topic tags:(NSArray*)tags block:(void (^)(AlivcWebSocketInfoModel *infoModel,NSError *error))block
{
    NSDictionary *param = @{@"topic":topic,
                            @"subscriptionName":topic};
    [[AlivcRequestManager manager] requestWithHost:@"mns/topic/websocket/info" param:param block:^(NSDictionary *response, NSError *error) {
        
        if (error) {
            if(block)
                block(nil,error);
            return ;
        }
        
        AlivcWebSocketInfoModel *infoModel = [[AlivcWebSocketInfoModel alloc] initWithDictionary: response];
        infoModel.tags = tags;
        infoModel.topicName = topic;
        if(block)
            block(infoModel,error);
    }];
}

+(void)quitChatRoomSuccess:(void (^)())successBlock
                     error:(void (^)(NSError *error))errorBlock
{
    [[AlivcLiveClient shareClient] quitChatRoomSuccess:^{
        if (successBlock) {
            successBlock();
        }
    } error:^(NSError *error) {
        if(errorBlock)
            errorBlock(error);
    }];
}

+(void)joinChatRoomWithWebSocketInfoModel:(AlivcWebSocketInfoModel *)model
                                  success:(void (^)())successBlock
                                    error:(void (^)(NSError *error))errorBlock
{
    [[AlivcLiveClient shareClient] joinChatRoomWithWebSocketInfoModel:model success:^{
        if(successBlock)
            successBlock();
    } error:^(NSError *error) {
        if (errorBlock) {
            errorBlock(error);
        }
    }];
}

@end
